package utf8

// THIS FILE WAS AUTO-GENERATED. DO NOT MODIFY.

const (
	groundState   State = 0
	tail3State    State = 1
	tail2State    State = 2
	tail1State    State = 3
	u32e0State    State = 4
	u32edState    State = 5
	utf843f0State State = 6
	utf843f4State State = 7

	invalidSequenceAction Action = 0
	emitByteAction        Action = 1
	setByte1Action        Action = 2
	setByte2Action        Action = 3
	setByte2TopAction     Action = 4
	setByte3Action        Action = 5
	setByte3TopAction     Action = 6
	setByte4Action        Action = 7
)

var (
	stateNames = []string{
		"groundState",
		"tail3State",
		"tail2State",
		"tail1State",
		"u32e0State",
		"u32edState",
		"utf843f0State",
		"utf843f4State",
	}

	stateTable = [][]byte{
		{ //State groundState = 0
			groundState | (emitByteAction << 4),        // 00
			groundState | (emitByteAction << 4),        // 01
			groundState | (emitByteAction << 4),        // 02
			groundState | (emitByteAction << 4),        // 03
			groundState | (emitByteAction << 4),        // 04
			groundState | (emitByteAction << 4),        // 05
			groundState | (emitByteAction << 4),        // 06
			groundState | (emitByteAction << 4),        // 07
			groundState | (emitByteAction << 4),        // 08
			groundState | (emitByteAction << 4),        // 09
			groundState | (emitByteAction << 4),        // 0a
			groundState | (emitByteAction << 4),        // 0b
			groundState | (emitByteAction << 4),        // 0c
			groundState | (emitByteAction << 4),        // 0d
			groundState | (emitByteAction << 4),        // 0e
			groundState | (emitByteAction << 4),        // 0f
			groundState | (emitByteAction << 4),        // 10
			groundState | (emitByteAction << 4),        // 11
			groundState | (emitByteAction << 4),        // 12
			groundState | (emitByteAction << 4),        // 13
			groundState | (emitByteAction << 4),        // 14
			groundState | (emitByteAction << 4),        // 15
			groundState | (emitByteAction << 4),        // 16
			groundState | (emitByteAction << 4),        // 17
			groundState | (emitByteAction << 4),        // 18
			groundState | (emitByteAction << 4),        // 19
			groundState | (emitByteAction << 4),        // 1a
			groundState | (emitByteAction << 4),        // 1b
			groundState | (emitByteAction << 4),        // 1c
			groundState | (emitByteAction << 4),        // 1d
			groundState | (emitByteAction << 4),        // 1e
			groundState | (emitByteAction << 4),        // 1f
			groundState | (emitByteAction << 4),        // 20
			groundState | (emitByteAction << 4),        // 21
			groundState | (emitByteAction << 4),        // 22
			groundState | (emitByteAction << 4),        // 23
			groundState | (emitByteAction << 4),        // 24
			groundState | (emitByteAction << 4),        // 25
			groundState | (emitByteAction << 4),        // 26
			groundState | (emitByteAction << 4),        // 27
			groundState | (emitByteAction << 4),        // 28
			groundState | (emitByteAction << 4),        // 29
			groundState | (emitByteAction << 4),        // 2a
			groundState | (emitByteAction << 4),        // 2b
			groundState | (emitByteAction << 4),        // 2c
			groundState | (emitByteAction << 4),        // 2d
			groundState | (emitByteAction << 4),        // 2e
			groundState | (emitByteAction << 4),        // 2f
			groundState | (emitByteAction << 4),        // 30
			groundState | (emitByteAction << 4),        // 31
			groundState | (emitByteAction << 4),        // 32
			groundState | (emitByteAction << 4),        // 33
			groundState | (emitByteAction << 4),        // 34
			groundState | (emitByteAction << 4),        // 35
			groundState | (emitByteAction << 4),        // 36
			groundState | (emitByteAction << 4),        // 37
			groundState | (emitByteAction << 4),        // 38
			groundState | (emitByteAction << 4),        // 39
			groundState | (emitByteAction << 4),        // 3a
			groundState | (emitByteAction << 4),        // 3b
			groundState | (emitByteAction << 4),        // 3c
			groundState | (emitByteAction << 4),        // 3d
			groundState | (emitByteAction << 4),        // 3e
			groundState | (emitByteAction << 4),        // 3f
			groundState | (emitByteAction << 4),        // 40
			groundState | (emitByteAction << 4),        // 41
			groundState | (emitByteAction << 4),        // 42
			groundState | (emitByteAction << 4),        // 43
			groundState | (emitByteAction << 4),        // 44
			groundState | (emitByteAction << 4),        // 45
			groundState | (emitByteAction << 4),        // 46
			groundState | (emitByteAction << 4),        // 47
			groundState | (emitByteAction << 4),        // 48
			groundState | (emitByteAction << 4),        // 49
			groundState | (emitByteAction << 4),        // 4a
			groundState | (emitByteAction << 4),        // 4b
			groundState | (emitByteAction << 4),        // 4c
			groundState | (emitByteAction << 4),        // 4d
			groundState | (emitByteAction << 4),        // 4e
			groundState | (emitByteAction << 4),        // 4f
			groundState | (emitByteAction << 4),        // 50
			groundState | (emitByteAction << 4),        // 51
			groundState | (emitByteAction << 4),        // 52
			groundState | (emitByteAction << 4),        // 53
			groundState | (emitByteAction << 4),        // 54
			groundState | (emitByteAction << 4),        // 55
			groundState | (emitByteAction << 4),        // 56
			groundState | (emitByteAction << 4),        // 57
			groundState | (emitByteAction << 4),        // 58
			groundState | (emitByteAction << 4),        // 59
			groundState | (emitByteAction << 4),        // 5a
			groundState | (emitByteAction << 4),        // 5b
			groundState | (emitByteAction << 4),        // 5c
			groundState | (emitByteAction << 4),        // 5d
			groundState | (emitByteAction << 4),        // 5e
			groundState | (emitByteAction << 4),        // 5f
			groundState | (emitByteAction << 4),        // 60
			groundState | (emitByteAction << 4),        // 61
			groundState | (emitByteAction << 4),        // 62
			groundState | (emitByteAction << 4),        // 63
			groundState | (emitByteAction << 4),        // 64
			groundState | (emitByteAction << 4),        // 65
			groundState | (emitByteAction << 4),        // 66
			groundState | (emitByteAction << 4),        // 67
			groundState | (emitByteAction << 4),        // 68
			groundState | (emitByteAction << 4),        // 69
			groundState | (emitByteAction << 4),        // 6a
			groundState | (emitByteAction << 4),        // 6b
			groundState | (emitByteAction << 4),        // 6c
			groundState | (emitByteAction << 4),        // 6d
			groundState | (emitByteAction << 4),        // 6e
			groundState | (emitByteAction << 4),        // 6f
			groundState | (emitByteAction << 4),        // 70
			groundState | (emitByteAction << 4),        // 71
			groundState | (emitByteAction << 4),        // 72
			groundState | (emitByteAction << 4),        // 73
			groundState | (emitByteAction << 4),        // 74
			groundState | (emitByteAction << 4),        // 75
			groundState | (emitByteAction << 4),        // 76
			groundState | (emitByteAction << 4),        // 77
			groundState | (emitByteAction << 4),        // 78
			groundState | (emitByteAction << 4),        // 79
			groundState | (emitByteAction << 4),        // 7a
			groundState | (emitByteAction << 4),        // 7b
			groundState | (emitByteAction << 4),        // 7c
			groundState | (emitByteAction << 4),        // 7d
			groundState | (emitByteAction << 4),        // 7e
			groundState | (emitByteAction << 4),        // 7f
			groundState | (invalidSequenceAction << 4), // 80
			groundState | (invalidSequenceAction << 4), // 81
			groundState | (invalidSequenceAction << 4), // 82
			groundState | (invalidSequenceAction << 4), // 83
			groundState | (invalidSequenceAction << 4), // 84
			groundState | (invalidSequenceAction << 4), // 85
			groundState | (invalidSequenceAction << 4), // 86
			groundState | (invalidSequenceAction << 4), // 87
			groundState | (invalidSequenceAction << 4), // 88
			groundState | (invalidSequenceAction << 4), // 89
			groundState | (invalidSequenceAction << 4), // 8a
			groundState | (invalidSequenceAction << 4), // 8b
			groundState | (invalidSequenceAction << 4), // 8c
			groundState | (invalidSequenceAction << 4), // 8d
			groundState | (invalidSequenceAction << 4), // 8e
			groundState | (invalidSequenceAction << 4), // 8f
			groundState | (invalidSequenceAction << 4), // 90
			groundState | (invalidSequenceAction << 4), // 91
			groundState | (invalidSequenceAction << 4), // 92
			groundState | (invalidSequenceAction << 4), // 93
			groundState | (invalidSequenceAction << 4), // 94
			groundState | (invalidSequenceAction << 4), // 95
			groundState | (invalidSequenceAction << 4), // 96
			groundState | (invalidSequenceAction << 4), // 97
			groundState | (invalidSequenceAction << 4), // 98
			groundState | (invalidSequenceAction << 4), // 99
			groundState | (invalidSequenceAction << 4), // 9a
			groundState | (invalidSequenceAction << 4), // 9b
			groundState | (invalidSequenceAction << 4), // 9c
			groundState | (invalidSequenceAction << 4), // 9d
			groundState | (invalidSequenceAction << 4), // 9e
			groundState | (invalidSequenceAction << 4), // 9f
			groundState | (invalidSequenceAction << 4), // a0
			groundState | (invalidSequenceAction << 4), // a1
			groundState | (invalidSequenceAction << 4), // a2
			groundState | (invalidSequenceAction << 4), // a3
			groundState | (invalidSequenceAction << 4), // a4
			groundState | (invalidSequenceAction << 4), // a5
			groundState | (invalidSequenceAction << 4), // a6
			groundState | (invalidSequenceAction << 4), // a7
			groundState | (invalidSequenceAction << 4), // a8
			groundState | (invalidSequenceAction << 4), // a9
			groundState | (invalidSequenceAction << 4), // aa
			groundState | (invalidSequenceAction << 4), // ab
			groundState | (invalidSequenceAction << 4), // ac
			groundState | (invalidSequenceAction << 4), // ad
			groundState | (invalidSequenceAction << 4), // ae
			groundState | (invalidSequenceAction << 4), // af
			groundState | (invalidSequenceAction << 4), // b0
			groundState | (invalidSequenceAction << 4), // b1
			groundState | (invalidSequenceAction << 4), // b2
			groundState | (invalidSequenceAction << 4), // b3
			groundState | (invalidSequenceAction << 4), // b4
			groundState | (invalidSequenceAction << 4), // b5
			groundState | (invalidSequenceAction << 4), // b6
			groundState | (invalidSequenceAction << 4), // b7
			groundState | (invalidSequenceAction << 4), // b8
			groundState | (invalidSequenceAction << 4), // b9
			groundState | (invalidSequenceAction << 4), // ba
			groundState | (invalidSequenceAction << 4), // bb
			groundState | (invalidSequenceAction << 4), // bc
			groundState | (invalidSequenceAction << 4), // bd
			groundState | (invalidSequenceAction << 4), // be
			groundState | (invalidSequenceAction << 4), // bf
			groundState | (invalidSequenceAction << 4), // c0
			groundState | (invalidSequenceAction << 4), // c1
			tail1State | (setByte2TopAction << 4),      // c2
			tail1State | (setByte2TopAction << 4),      // c3
			tail1State | (setByte2TopAction << 4),      // c4
			tail1State | (setByte2TopAction << 4),      // c5
			tail1State | (setByte2TopAction << 4),      // c6
			tail1State | (setByte2TopAction << 4),      // c7
			tail1State | (setByte2TopAction << 4),      // c8
			tail1State | (setByte2TopAction << 4),      // c9
			tail1State | (setByte2TopAction << 4),      // ca
			tail1State | (setByte2TopAction << 4),      // cb
			tail1State | (setByte2TopAction << 4),      // cc
			tail1State | (setByte2TopAction << 4),      // cd
			tail1State | (setByte2TopAction << 4),      // ce
			tail1State | (setByte2TopAction << 4),      // cf
			tail1State | (setByte2TopAction << 4),      // d0
			tail1State | (setByte2TopAction << 4),      // d1
			tail1State | (setByte2TopAction << 4),      // d2
			tail1State | (setByte2TopAction << 4),      // d3
			tail1State | (setByte2TopAction << 4),      // d4
			tail1State | (setByte2TopAction << 4),      // d5
			tail1State | (setByte2TopAction << 4),      // d6
			tail1State | (setByte2TopAction << 4),      // d7
			tail1State | (setByte2TopAction << 4),      // d8
			tail1State | (setByte2TopAction << 4),      // d9
			tail1State | (setByte2TopAction << 4),      // da
			tail1State | (setByte2TopAction << 4),      // db
			tail1State | (setByte2TopAction << 4),      // dc
			tail1State | (setByte2TopAction << 4),      // dd
			tail1State | (setByte2TopAction << 4),      // de
			tail1State | (setByte2TopAction << 4),      // df
			u32e0State | (setByte3TopAction << 4),      // e0
			tail2State | (setByte3TopAction << 4),      // e1
			tail2State | (setByte3TopAction << 4),      // e2
			tail2State | (setByte3TopAction << 4),      // e3
			tail2State | (setByte3TopAction << 4),      // e4
			tail2State | (setByte3TopAction << 4),      // e5
			tail2State | (setByte3TopAction << 4),      // e6
			tail2State | (setByte3TopAction << 4),      // e7
			tail2State | (setByte3TopAction << 4),      // e8
			tail2State | (setByte3TopAction << 4),      // e9
			tail2State | (setByte3TopAction << 4),      // ea
			tail2State | (setByte3TopAction << 4),      // eb
			tail2State | (setByte3TopAction << 4),      // ec
			u32edState | (setByte3TopAction << 4),      // ed
			tail2State | (setByte3TopAction << 4),      // ee
			tail2State | (setByte3TopAction << 4),      // ef
			utf843f0State | (setByte4Action << 4),      // f0
			tail3State | (setByte4Action << 4),         // f1
			tail3State | (setByte4Action << 4),         // f2
			tail3State | (setByte4Action << 4),         // f3
			utf843f4State | (setByte4Action << 4),      // f4
			groundState | (invalidSequenceAction << 4), // f5
			groundState | (invalidSequenceAction << 4), // f6
			groundState | (invalidSequenceAction << 4), // f7
			groundState | (invalidSequenceAction << 4), // f8
			groundState | (invalidSequenceAction << 4), // f9
			groundState | (invalidSequenceAction << 4), // fa
			groundState | (invalidSequenceAction << 4), // fb
			groundState | (invalidSequenceAction << 4), // fc
			groundState | (invalidSequenceAction << 4), // fd
			groundState | (invalidSequenceAction << 4), // fe
			groundState | (invalidSequenceAction << 4), // ff
		},
		{ //State tail3State = 1
			groundState | (invalidSequenceAction << 4), // 00
			groundState | (invalidSequenceAction << 4), // 01
			groundState | (invalidSequenceAction << 4), // 02
			groundState | (invalidSequenceAction << 4), // 03
			groundState | (invalidSequenceAction << 4), // 04
			groundState | (invalidSequenceAction << 4), // 05
			groundState | (invalidSequenceAction << 4), // 06
			groundState | (invalidSequenceAction << 4), // 07
			groundState | (invalidSequenceAction << 4), // 08
			groundState | (invalidSequenceAction << 4), // 09
			groundState | (invalidSequenceAction << 4), // 0a
			groundState | (invalidSequenceAction << 4), // 0b
			groundState | (invalidSequenceAction << 4), // 0c
			groundState | (invalidSequenceAction << 4), // 0d
			groundState | (invalidSequenceAction << 4), // 0e
			groundState | (invalidSequenceAction << 4), // 0f
			groundState | (invalidSequenceAction << 4), // 10
			groundState | (invalidSequenceAction << 4), // 11
			groundState | (invalidSequenceAction << 4), // 12
			groundState | (invalidSequenceAction << 4), // 13
			groundState | (invalidSequenceAction << 4), // 14
			groundState | (invalidSequenceAction << 4), // 15
			groundState | (invalidSequenceAction << 4), // 16
			groundState | (invalidSequenceAction << 4), // 17
			groundState | (invalidSequenceAction << 4), // 18
			groundState | (invalidSequenceAction << 4), // 19
			groundState | (invalidSequenceAction << 4), // 1a
			groundState | (invalidSequenceAction << 4), // 1b
			groundState | (invalidSequenceAction << 4), // 1c
			groundState | (invalidSequenceAction << 4), // 1d
			groundState | (invalidSequenceAction << 4), // 1e
			groundState | (invalidSequenceAction << 4), // 1f
			groundState | (invalidSequenceAction << 4), // 20
			groundState | (invalidSequenceAction << 4), // 21
			groundState | (invalidSequenceAction << 4), // 22
			groundState | (invalidSequenceAction << 4), // 23
			groundState | (invalidSequenceAction << 4), // 24
			groundState | (invalidSequenceAction << 4), // 25
			groundState | (invalidSequenceAction << 4), // 26
			groundState | (invalidSequenceAction << 4), // 27
			groundState | (invalidSequenceAction << 4), // 28
			groundState | (invalidSequenceAction << 4), // 29
			groundState | (invalidSequenceAction << 4), // 2a
			groundState | (invalidSequenceAction << 4), // 2b
			groundState | (invalidSequenceAction << 4), // 2c
			groundState | (invalidSequenceAction << 4), // 2d
			groundState | (invalidSequenceAction << 4), // 2e
			groundState | (invalidSequenceAction << 4), // 2f
			groundState | (invalidSequenceAction << 4), // 30
			groundState | (invalidSequenceAction << 4), // 31
			groundState | (invalidSequenceAction << 4), // 32
			groundState | (invalidSequenceAction << 4), // 33
			groundState | (invalidSequenceAction << 4), // 34
			groundState | (invalidSequenceAction << 4), // 35
			groundState | (invalidSequenceAction << 4), // 36
			groundState | (invalidSequenceAction << 4), // 37
			groundState | (invalidSequenceAction << 4), // 38
			groundState | (invalidSequenceAction << 4), // 39
			groundState | (invalidSequenceAction << 4), // 3a
			groundState | (invalidSequenceAction << 4), // 3b
			groundState | (invalidSequenceAction << 4), // 3c
			groundState | (invalidSequenceAction << 4), // 3d
			groundState | (invalidSequenceAction << 4), // 3e
			groundState | (invalidSequenceAction << 4), // 3f
			groundState | (invalidSequenceAction << 4), // 40
			groundState | (invalidSequenceAction << 4), // 41
			groundState | (invalidSequenceAction << 4), // 42
			groundState | (invalidSequenceAction << 4), // 43
			groundState | (invalidSequenceAction << 4), // 44
			groundState | (invalidSequenceAction << 4), // 45
			groundState | (invalidSequenceAction << 4), // 46
			groundState | (invalidSequenceAction << 4), // 47
			groundState | (invalidSequenceAction << 4), // 48
			groundState | (invalidSequenceAction << 4), // 49
			groundState | (invalidSequenceAction << 4), // 4a
			groundState | (invalidSequenceAction << 4), // 4b
			groundState | (invalidSequenceAction << 4), // 4c
			groundState | (invalidSequenceAction << 4), // 4d
			groundState | (invalidSequenceAction << 4), // 4e
			groundState | (invalidSequenceAction << 4), // 4f
			groundState | (invalidSequenceAction << 4), // 50
			groundState | (invalidSequenceAction << 4), // 51
			groundState | (invalidSequenceAction << 4), // 52
			groundState | (invalidSequenceAction << 4), // 53
			groundState | (invalidSequenceAction << 4), // 54
			groundState | (invalidSequenceAction << 4), // 55
			groundState | (invalidSequenceAction << 4), // 56
			groundState | (invalidSequenceAction << 4), // 57
			groundState | (invalidSequenceAction << 4), // 58
			groundState | (invalidSequenceAction << 4), // 59
			groundState | (invalidSequenceAction << 4), // 5a
			groundState | (invalidSequenceAction << 4), // 5b
			groundState | (invalidSequenceAction << 4), // 5c
			groundState | (invalidSequenceAction << 4), // 5d
			groundState | (invalidSequenceAction << 4), // 5e
			groundState | (invalidSequenceAction << 4), // 5f
			groundState | (invalidSequenceAction << 4), // 60
			groundState | (invalidSequenceAction << 4), // 61
			groundState | (invalidSequenceAction << 4), // 62
			groundState | (invalidSequenceAction << 4), // 63
			groundState | (invalidSequenceAction << 4), // 64
			groundState | (invalidSequenceAction << 4), // 65
			groundState | (invalidSequenceAction << 4), // 66
			groundState | (invalidSequenceAction << 4), // 67
			groundState | (invalidSequenceAction << 4), // 68
			groundState | (invalidSequenceAction << 4), // 69
			groundState | (invalidSequenceAction << 4), // 6a
			groundState | (invalidSequenceAction << 4), // 6b
			groundState | (invalidSequenceAction << 4), // 6c
			groundState | (invalidSequenceAction << 4), // 6d
			groundState | (invalidSequenceAction << 4), // 6e
			groundState | (invalidSequenceAction << 4), // 6f
			groundState | (invalidSequenceAction << 4), // 70
			groundState | (invalidSequenceAction << 4), // 71
			groundState | (invalidSequenceAction << 4), // 72
			groundState | (invalidSequenceAction << 4), // 73
			groundState | (invalidSequenceAction << 4), // 74
			groundState | (invalidSequenceAction << 4), // 75
			groundState | (invalidSequenceAction << 4), // 76
			groundState | (invalidSequenceAction << 4), // 77
			groundState | (invalidSequenceAction << 4), // 78
			groundState | (invalidSequenceAction << 4), // 79
			groundState | (invalidSequenceAction << 4), // 7a
			groundState | (invalidSequenceAction << 4), // 7b
			groundState | (invalidSequenceAction << 4), // 7c
			groundState | (invalidSequenceAction << 4), // 7d
			groundState | (invalidSequenceAction << 4), // 7e
			groundState | (invalidSequenceAction << 4), // 7f
			tail2State | (setByte3Action << 4),         // 80
			tail2State | (setByte3Action << 4),         // 81
			tail2State | (setByte3Action << 4),         // 82
			tail2State | (setByte3Action << 4),         // 83
			tail2State | (setByte3Action << 4),         // 84
			tail2State | (setByte3Action << 4),         // 85
			tail2State | (setByte3Action << 4),         // 86
			tail2State | (setByte3Action << 4),         // 87
			tail2State | (setByte3Action << 4),         // 88
			tail2State | (setByte3Action << 4),         // 89
			tail2State | (setByte3Action << 4),         // 8a
			tail2State | (setByte3Action << 4),         // 8b
			tail2State | (setByte3Action << 4),         // 8c
			tail2State | (setByte3Action << 4),         // 8d
			tail2State | (setByte3Action << 4),         // 8e
			tail2State | (setByte3Action << 4),         // 8f
			tail2State | (setByte3Action << 4),         // 90
			tail2State | (setByte3Action << 4),         // 91
			tail2State | (setByte3Action << 4),         // 92
			tail2State | (setByte3Action << 4),         // 93
			tail2State | (setByte3Action << 4),         // 94
			tail2State | (setByte3Action << 4),         // 95
			tail2State | (setByte3Action << 4),         // 96
			tail2State | (setByte3Action << 4),         // 97
			tail2State | (setByte3Action << 4),         // 98
			tail2State | (setByte3Action << 4),         // 99
			tail2State | (setByte3Action << 4),         // 9a
			tail2State | (setByte3Action << 4),         // 9b
			tail2State | (setByte3Action << 4),         // 9c
			tail2State | (setByte3Action << 4),         // 9d
			tail2State | (setByte3Action << 4),         // 9e
			tail2State | (setByte3Action << 4),         // 9f
			tail2State | (setByte3Action << 4),         // a0
			tail2State | (setByte3Action << 4),         // a1
			tail2State | (setByte3Action << 4),         // a2
			tail2State | (setByte3Action << 4),         // a3
			tail2State | (setByte3Action << 4),         // a4
			tail2State | (setByte3Action << 4),         // a5
			tail2State | (setByte3Action << 4),         // a6
			tail2State | (setByte3Action << 4),         // a7
			tail2State | (setByte3Action << 4),         // a8
			tail2State | (setByte3Action << 4),         // a9
			tail2State | (setByte3Action << 4),         // aa
			tail2State | (setByte3Action << 4),         // ab
			tail2State | (setByte3Action << 4),         // ac
			tail2State | (setByte3Action << 4),         // ad
			tail2State | (setByte3Action << 4),         // ae
			tail2State | (setByte3Action << 4),         // af
			tail2State | (setByte3Action << 4),         // b0
			tail2State | (setByte3Action << 4),         // b1
			tail2State | (setByte3Action << 4),         // b2
			tail2State | (setByte3Action << 4),         // b3
			tail2State | (setByte3Action << 4),         // b4
			tail2State | (setByte3Action << 4),         // b5
			tail2State | (setByte3Action << 4),         // b6
			tail2State | (setByte3Action << 4),         // b7
			tail2State | (setByte3Action << 4),         // b8
			tail2State | (setByte3Action << 4),         // b9
			tail2State | (setByte3Action << 4),         // ba
			tail2State | (setByte3Action << 4),         // bb
			tail2State | (setByte3Action << 4),         // bc
			tail2State | (setByte3Action << 4),         // bd
			tail2State | (setByte3Action << 4),         // be
			tail2State | (setByte3Action << 4),         // bf
			groundState | (invalidSequenceAction << 4), // c0
			groundState | (invalidSequenceAction << 4), // c1
			groundState | (invalidSequenceAction << 4), // c2
			groundState | (invalidSequenceAction << 4), // c3
			groundState | (invalidSequenceAction << 4), // c4
			groundState | (invalidSequenceAction << 4), // c5
			groundState | (invalidSequenceAction << 4), // c6
			groundState | (invalidSequenceAction << 4), // c7
			groundState | (invalidSequenceAction << 4), // c8
			groundState | (invalidSequenceAction << 4), // c9
			groundState | (invalidSequenceAction << 4), // ca
			groundState | (invalidSequenceAction << 4), // cb
			groundState | (invalidSequenceAction << 4), // cc
			groundState | (invalidSequenceAction << 4), // cd
			groundState | (invalidSequenceAction << 4), // ce
			groundState | (invalidSequenceAction << 4), // cf
			groundState | (invalidSequenceAction << 4), // d0
			groundState | (invalidSequenceAction << 4), // d1
			groundState | (invalidSequenceAction << 4), // d2
			groundState | (invalidSequenceAction << 4), // d3
			groundState | (invalidSequenceAction << 4), // d4
			groundState | (invalidSequenceAction << 4), // d5
			groundState | (invalidSequenceAction << 4), // d6
			groundState | (invalidSequenceAction << 4), // d7
			groundState | (invalidSequenceAction << 4), // d8
			groundState | (invalidSequenceAction << 4), // d9
			groundState | (invalidSequenceAction << 4), // da
			groundState | (invalidSequenceAction << 4), // db
			groundState | (invalidSequenceAction << 4), // dc
			groundState | (invalidSequenceAction << 4), // dd
			groundState | (invalidSequenceAction << 4), // de
			groundState | (invalidSequenceAction << 4), // df
			groundState | (invalidSequenceAction << 4), // e0
			groundState | (invalidSequenceAction << 4), // e1
			groundState | (invalidSequenceAction << 4), // e2
			groundState | (invalidSequenceAction << 4), // e3
			groundState | (invalidSequenceAction << 4), // e4
			groundState | (invalidSequenceAction << 4), // e5
			groundState | (invalidSequenceAction << 4), // e6
			groundState | (invalidSequenceAction << 4), // e7
			groundState | (invalidSequenceAction << 4), // e8
			groundState | (invalidSequenceAction << 4), // e9
			groundState | (invalidSequenceAction << 4), // ea
			groundState | (invalidSequenceAction << 4), // eb
			groundState | (invalidSequenceAction << 4), // ec
			groundState | (invalidSequenceAction << 4), // ed
			groundState | (invalidSequenceAction << 4), // ee
			groundState | (invalidSequenceAction << 4), // ef
			groundState | (invalidSequenceAction << 4), // f0
			groundState | (invalidSequenceAction << 4), // f1
			groundState | (invalidSequenceAction << 4), // f2
			groundState | (invalidSequenceAction << 4), // f3
			groundState | (invalidSequenceAction << 4), // f4
			groundState | (invalidSequenceAction << 4), // f5
			groundState | (invalidSequenceAction << 4), // f6
			groundState | (invalidSequenceAction << 4), // f7
			groundState | (invalidSequenceAction << 4), // f8
			groundState | (invalidSequenceAction << 4), // f9
			groundState | (invalidSequenceAction << 4), // fa
			groundState | (invalidSequenceAction << 4), // fb
			groundState | (invalidSequenceAction << 4), // fc
			groundState | (invalidSequenceAction << 4), // fd
			groundState | (invalidSequenceAction << 4), // fe
			groundState | (invalidSequenceAction << 4), // ff
		},
		{ //State tail2State = 2
			groundState | (invalidSequenceAction << 4), // 00
			groundState | (invalidSequenceAction << 4), // 01
			groundState | (invalidSequenceAction << 4), // 02
			groundState | (invalidSequenceAction << 4), // 03
			groundState | (invalidSequenceAction << 4), // 04
			groundState | (invalidSequenceAction << 4), // 05
			groundState | (invalidSequenceAction << 4), // 06
			groundState | (invalidSequenceAction << 4), // 07
			groundState | (invalidSequenceAction << 4), // 08
			groundState | (invalidSequenceAction << 4), // 09
			groundState | (invalidSequenceAction << 4), // 0a
			groundState | (invalidSequenceAction << 4), // 0b
			groundState | (invalidSequenceAction << 4), // 0c
			groundState | (invalidSequenceAction << 4), // 0d
			groundState | (invalidSequenceAction << 4), // 0e
			groundState | (invalidSequenceAction << 4), // 0f
			groundState | (invalidSequenceAction << 4), // 10
			groundState | (invalidSequenceAction << 4), // 11
			groundState | (invalidSequenceAction << 4), // 12
			groundState | (invalidSequenceAction << 4), // 13
			groundState | (invalidSequenceAction << 4), // 14
			groundState | (invalidSequenceAction << 4), // 15
			groundState | (invalidSequenceAction << 4), // 16
			groundState | (invalidSequenceAction << 4), // 17
			groundState | (invalidSequenceAction << 4), // 18
			groundState | (invalidSequenceAction << 4), // 19
			groundState | (invalidSequenceAction << 4), // 1a
			groundState | (invalidSequenceAction << 4), // 1b
			groundState | (invalidSequenceAction << 4), // 1c
			groundState | (invalidSequenceAction << 4), // 1d
			groundState | (invalidSequenceAction << 4), // 1e
			groundState | (invalidSequenceAction << 4), // 1f
			groundState | (invalidSequenceAction << 4), // 20
			groundState | (invalidSequenceAction << 4), // 21
			groundState | (invalidSequenceAction << 4), // 22
			groundState | (invalidSequenceAction << 4), // 23
			groundState | (invalidSequenceAction << 4), // 24
			groundState | (invalidSequenceAction << 4), // 25
			groundState | (invalidSequenceAction << 4), // 26
			groundState | (invalidSequenceAction << 4), // 27
			groundState | (invalidSequenceAction << 4), // 28
			groundState | (invalidSequenceAction << 4), // 29
			groundState | (invalidSequenceAction << 4), // 2a
			groundState | (invalidSequenceAction << 4), // 2b
			groundState | (invalidSequenceAction << 4), // 2c
			groundState | (invalidSequenceAction << 4), // 2d
			groundState | (invalidSequenceAction << 4), // 2e
			groundState | (invalidSequenceAction << 4), // 2f
			groundState | (invalidSequenceAction << 4), // 30
			groundState | (invalidSequenceAction << 4), // 31
			groundState | (invalidSequenceAction << 4), // 32
			groundState | (invalidSequenceAction << 4), // 33
			groundState | (invalidSequenceAction << 4), // 34
			groundState | (invalidSequenceAction << 4), // 35
			groundState | (invalidSequenceAction << 4), // 36
			groundState | (invalidSequenceAction << 4), // 37
			groundState | (invalidSequenceAction << 4), // 38
			groundState | (invalidSequenceAction << 4), // 39
			groundState | (invalidSequenceAction << 4), // 3a
			groundState | (invalidSequenceAction << 4), // 3b
			groundState | (invalidSequenceAction << 4), // 3c
			groundState | (invalidSequenceAction << 4), // 3d
			groundState | (invalidSequenceAction << 4), // 3e
			groundState | (invalidSequenceAction << 4), // 3f
			groundState | (invalidSequenceAction << 4), // 40
			groundState | (invalidSequenceAction << 4), // 41
			groundState | (invalidSequenceAction << 4), // 42
			groundState | (invalidSequenceAction << 4), // 43
			groundState | (invalidSequenceAction << 4), // 44
			groundState | (invalidSequenceAction << 4), // 45
			groundState | (invalidSequenceAction << 4), // 46
			groundState | (invalidSequenceAction << 4), // 47
			groundState | (invalidSequenceAction << 4), // 48
			groundState | (invalidSequenceAction << 4), // 49
			groundState | (invalidSequenceAction << 4), // 4a
			groundState | (invalidSequenceAction << 4), // 4b
			groundState | (invalidSequenceAction << 4), // 4c
			groundState | (invalidSequenceAction << 4), // 4d
			groundState | (invalidSequenceAction << 4), // 4e
			groundState | (invalidSequenceAction << 4), // 4f
			groundState | (invalidSequenceAction << 4), // 50
			groundState | (invalidSequenceAction << 4), // 51
			groundState | (invalidSequenceAction << 4), // 52
			groundState | (invalidSequenceAction << 4), // 53
			groundState | (invalidSequenceAction << 4), // 54
			groundState | (invalidSequenceAction << 4), // 55
			groundState | (invalidSequenceAction << 4), // 56
			groundState | (invalidSequenceAction << 4), // 57
			groundState | (invalidSequenceAction << 4), // 58
			groundState | (invalidSequenceAction << 4), // 59
			groundState | (invalidSequenceAction << 4), // 5a
			groundState | (invalidSequenceAction << 4), // 5b
			groundState | (invalidSequenceAction << 4), // 5c
			groundState | (invalidSequenceAction << 4), // 5d
			groundState | (invalidSequenceAction << 4), // 5e
			groundState | (invalidSequenceAction << 4), // 5f
			groundState | (invalidSequenceAction << 4), // 60
			groundState | (invalidSequenceAction << 4), // 61
			groundState | (invalidSequenceAction << 4), // 62
			groundState | (invalidSequenceAction << 4), // 63
			groundState | (invalidSequenceAction << 4), // 64
			groundState | (invalidSequenceAction << 4), // 65
			groundState | (invalidSequenceAction << 4), // 66
			groundState | (invalidSequenceAction << 4), // 67
			groundState | (invalidSequenceAction << 4), // 68
			groundState | (invalidSequenceAction << 4), // 69
			groundState | (invalidSequenceAction << 4), // 6a
			groundState | (invalidSequenceAction << 4), // 6b
			groundState | (invalidSequenceAction << 4), // 6c
			groundState | (invalidSequenceAction << 4), // 6d
			groundState | (invalidSequenceAction << 4), // 6e
			groundState | (invalidSequenceAction << 4), // 6f
			groundState | (invalidSequenceAction << 4), // 70
			groundState | (invalidSequenceAction << 4), // 71
			groundState | (invalidSequenceAction << 4), // 72
			groundState | (invalidSequenceAction << 4), // 73
			groundState | (invalidSequenceAction << 4), // 74
			groundState | (invalidSequenceAction << 4), // 75
			groundState | (invalidSequenceAction << 4), // 76
			groundState | (invalidSequenceAction << 4), // 77
			groundState | (invalidSequenceAction << 4), // 78
			groundState | (invalidSequenceAction << 4), // 79
			groundState | (invalidSequenceAction << 4), // 7a
			groundState | (invalidSequenceAction << 4), // 7b
			groundState | (invalidSequenceAction << 4), // 7c
			groundState | (invalidSequenceAction << 4), // 7d
			groundState | (invalidSequenceAction << 4), // 7e
			groundState | (invalidSequenceAction << 4), // 7f
			tail1State | (setByte2Action << 4),         // 80
			tail1State | (setByte2Action << 4),         // 81
			tail1State | (setByte2Action << 4),         // 82
			tail1State | (setByte2Action << 4),         // 83
			tail1State | (setByte2Action << 4),         // 84
			tail1State | (setByte2Action << 4),         // 85
			tail1State | (setByte2Action << 4),         // 86
			tail1State | (setByte2Action << 4),         // 87
			tail1State | (setByte2Action << 4),         // 88
			tail1State | (setByte2Action << 4),         // 89
			tail1State | (setByte2Action << 4),         // 8a
			tail1State | (setByte2Action << 4),         // 8b
			tail1State | (setByte2Action << 4),         // 8c
			tail1State | (setByte2Action << 4),         // 8d
			tail1State | (setByte2Action << 4),         // 8e
			tail1State | (setByte2Action << 4),         // 8f
			tail1State | (setByte2Action << 4),         // 90
			tail1State | (setByte2Action << 4),         // 91
			tail1State | (setByte2Action << 4),         // 92
			tail1State | (setByte2Action << 4),         // 93
			tail1State | (setByte2Action << 4),         // 94
			tail1State | (setByte2Action << 4),         // 95
			tail1State | (setByte2Action << 4),         // 96
			tail1State | (setByte2Action << 4),         // 97
			tail1State | (setByte2Action << 4),         // 98
			tail1State | (setByte2Action << 4),         // 99
			tail1State | (setByte2Action << 4),         // 9a
			tail1State | (setByte2Action << 4),         // 9b
			tail1State | (setByte2Action << 4),         // 9c
			tail1State | (setByte2Action << 4),         // 9d
			tail1State | (setByte2Action << 4),         // 9e
			tail1State | (setByte2Action << 4),         // 9f
			tail1State | (setByte2Action << 4),         // a0
			tail1State | (setByte2Action << 4),         // a1
			tail1State | (setByte2Action << 4),         // a2
			tail1State | (setByte2Action << 4),         // a3
			tail1State | (setByte2Action << 4),         // a4
			tail1State | (setByte2Action << 4),         // a5
			tail1State | (setByte2Action << 4),         // a6
			tail1State | (setByte2Action << 4),         // a7
			tail1State | (setByte2Action << 4),         // a8
			tail1State | (setByte2Action << 4),         // a9
			tail1State | (setByte2Action << 4),         // aa
			tail1State | (setByte2Action << 4),         // ab
			tail1State | (setByte2Action << 4),         // ac
			tail1State | (setByte2Action << 4),         // ad
			tail1State | (setByte2Action << 4),         // ae
			tail1State | (setByte2Action << 4),         // af
			tail1State | (setByte2Action << 4),         // b0
			tail1State | (setByte2Action << 4),         // b1
			tail1State | (setByte2Action << 4),         // b2
			tail1State | (setByte2Action << 4),         // b3
			tail1State | (setByte2Action << 4),         // b4
			tail1State | (setByte2Action << 4),         // b5
			tail1State | (setByte2Action << 4),         // b6
			tail1State | (setByte2Action << 4),         // b7
			tail1State | (setByte2Action << 4),         // b8
			tail1State | (setByte2Action << 4),         // b9
			tail1State | (setByte2Action << 4),         // ba
			tail1State | (setByte2Action << 4),         // bb
			tail1State | (setByte2Action << 4),         // bc
			tail1State | (setByte2Action << 4),         // bd
			tail1State | (setByte2Action << 4),         // be
			tail1State | (setByte2Action << 4),         // bf
			groundState | (invalidSequenceAction << 4), // c0
			groundState | (invalidSequenceAction << 4), // c1
			groundState | (invalidSequenceAction << 4), // c2
			groundState | (invalidSequenceAction << 4), // c3
			groundState | (invalidSequenceAction << 4), // c4
			groundState | (invalidSequenceAction << 4), // c5
			groundState | (invalidSequenceAction << 4), // c6
			groundState | (invalidSequenceAction << 4), // c7
			groundState | (invalidSequenceAction << 4), // c8
			groundState | (invalidSequenceAction << 4), // c9
			groundState | (invalidSequenceAction << 4), // ca
			groundState | (invalidSequenceAction << 4), // cb
			groundState | (invalidSequenceAction << 4), // cc
			groundState | (invalidSequenceAction << 4), // cd
			groundState | (invalidSequenceAction << 4), // ce
			groundState | (invalidSequenceAction << 4), // cf
			groundState | (invalidSequenceAction << 4), // d0
			groundState | (invalidSequenceAction << 4), // d1
			groundState | (invalidSequenceAction << 4), // d2
			groundState | (invalidSequenceAction << 4), // d3
			groundState | (invalidSequenceAction << 4), // d4
			groundState | (invalidSequenceAction << 4), // d5
			groundState | (invalidSequenceAction << 4), // d6
			groundState | (invalidSequenceAction << 4), // d7
			groundState | (invalidSequenceAction << 4), // d8
			groundState | (invalidSequenceAction << 4), // d9
			groundState | (invalidSequenceAction << 4), // da
			groundState | (invalidSequenceAction << 4), // db
			groundState | (invalidSequenceAction << 4), // dc
			groundState | (invalidSequenceAction << 4), // dd
			groundState | (invalidSequenceAction << 4), // de
			groundState | (invalidSequenceAction << 4), // df
			groundState | (invalidSequenceAction << 4), // e0
			groundState | (invalidSequenceAction << 4), // e1
			groundState | (invalidSequenceAction << 4), // e2
			groundState | (invalidSequenceAction << 4), // e3
			groundState | (invalidSequenceAction << 4), // e4
			groundState | (invalidSequenceAction << 4), // e5
			groundState | (invalidSequenceAction << 4), // e6
			groundState | (invalidSequenceAction << 4), // e7
			groundState | (invalidSequenceAction << 4), // e8
			groundState | (invalidSequenceAction << 4), // e9
			groundState | (invalidSequenceAction << 4), // ea
			groundState | (invalidSequenceAction << 4), // eb
			groundState | (invalidSequenceAction << 4), // ec
			groundState | (invalidSequenceAction << 4), // ed
			groundState | (invalidSequenceAction << 4), // ee
			groundState | (invalidSequenceAction << 4), // ef
			groundState | (invalidSequenceAction << 4), // f0
			groundState | (invalidSequenceAction << 4), // f1
			groundState | (invalidSequenceAction << 4), // f2
			groundState | (invalidSequenceAction << 4), // f3
			groundState | (invalidSequenceAction << 4), // f4
			groundState | (invalidSequenceAction << 4), // f5
			groundState | (invalidSequenceAction << 4), // f6
			groundState | (invalidSequenceAction << 4), // f7
			groundState | (invalidSequenceAction << 4), // f8
			groundState | (invalidSequenceAction << 4), // f9
			groundState | (invalidSequenceAction << 4), // fa
			groundState | (invalidSequenceAction << 4), // fb
			groundState | (invalidSequenceAction << 4), // fc
			groundState | (invalidSequenceAction << 4), // fd
			groundState | (invalidSequenceAction << 4), // fe
			groundState | (invalidSequenceAction << 4), // ff
		},
		{ //State tail1State = 3
			groundState | (invalidSequenceAction << 4), // 00
			groundState | (invalidSequenceAction << 4), // 01
			groundState | (invalidSequenceAction << 4), // 02
			groundState | (invalidSequenceAction << 4), // 03
			groundState | (invalidSequenceAction << 4), // 04
			groundState | (invalidSequenceAction << 4), // 05
			groundState | (invalidSequenceAction << 4), // 06
			groundState | (invalidSequenceAction << 4), // 07
			groundState | (invalidSequenceAction << 4), // 08
			groundState | (invalidSequenceAction << 4), // 09
			groundState | (invalidSequenceAction << 4), // 0a
			groundState | (invalidSequenceAction << 4), // 0b
			groundState | (invalidSequenceAction << 4), // 0c
			groundState | (invalidSequenceAction << 4), // 0d
			groundState | (invalidSequenceAction << 4), // 0e
			groundState | (invalidSequenceAction << 4), // 0f
			groundState | (invalidSequenceAction << 4), // 10
			groundState | (invalidSequenceAction << 4), // 11
			groundState | (invalidSequenceAction << 4), // 12
			groundState | (invalidSequenceAction << 4), // 13
			groundState | (invalidSequenceAction << 4), // 14
			groundState | (invalidSequenceAction << 4), // 15
			groundState | (invalidSequenceAction << 4), // 16
			groundState | (invalidSequenceAction << 4), // 17
			groundState | (invalidSequenceAction << 4), // 18
			groundState | (invalidSequenceAction << 4), // 19
			groundState | (invalidSequenceAction << 4), // 1a
			groundState | (invalidSequenceAction << 4), // 1b
			groundState | (invalidSequenceAction << 4), // 1c
			groundState | (invalidSequenceAction << 4), // 1d
			groundState | (invalidSequenceAction << 4), // 1e
			groundState | (invalidSequenceAction << 4), // 1f
			groundState | (invalidSequenceAction << 4), // 20
			groundState | (invalidSequenceAction << 4), // 21
			groundState | (invalidSequenceAction << 4), // 22
			groundState | (invalidSequenceAction << 4), // 23
			groundState | (invalidSequenceAction << 4), // 24
			groundState | (invalidSequenceAction << 4), // 25
			groundState | (invalidSequenceAction << 4), // 26
			groundState | (invalidSequenceAction << 4), // 27
			groundState | (invalidSequenceAction << 4), // 28
			groundState | (invalidSequenceAction << 4), // 29
			groundState | (invalidSequenceAction << 4), // 2a
			groundState | (invalidSequenceAction << 4), // 2b
			groundState | (invalidSequenceAction << 4), // 2c
			groundState | (invalidSequenceAction << 4), // 2d
			groundState | (invalidSequenceAction << 4), // 2e
			groundState | (invalidSequenceAction << 4), // 2f
			groundState | (invalidSequenceAction << 4), // 30
			groundState | (invalidSequenceAction << 4), // 31
			groundState | (invalidSequenceAction << 4), // 32
			groundState | (invalidSequenceAction << 4), // 33
			groundState | (invalidSequenceAction << 4), // 34
			groundState | (invalidSequenceAction << 4), // 35
			groundState | (invalidSequenceAction << 4), // 36
			groundState | (invalidSequenceAction << 4), // 37
			groundState | (invalidSequenceAction << 4), // 38
			groundState | (invalidSequenceAction << 4), // 39
			groundState | (invalidSequenceAction << 4), // 3a
			groundState | (invalidSequenceAction << 4), // 3b
			groundState | (invalidSequenceAction << 4), // 3c
			groundState | (invalidSequenceAction << 4), // 3d
			groundState | (invalidSequenceAction << 4), // 3e
			groundState | (invalidSequenceAction << 4), // 3f
			groundState | (invalidSequenceAction << 4), // 40
			groundState | (invalidSequenceAction << 4), // 41
			groundState | (invalidSequenceAction << 4), // 42
			groundState | (invalidSequenceAction << 4), // 43
			groundState | (invalidSequenceAction << 4), // 44
			groundState | (invalidSequenceAction << 4), // 45
			groundState | (invalidSequenceAction << 4), // 46
			groundState | (invalidSequenceAction << 4), // 47
			groundState | (invalidSequenceAction << 4), // 48
			groundState | (invalidSequenceAction << 4), // 49
			groundState | (invalidSequenceAction << 4), // 4a
			groundState | (invalidSequenceAction << 4), // 4b
			groundState | (invalidSequenceAction << 4), // 4c
			groundState | (invalidSequenceAction << 4), // 4d
			groundState | (invalidSequenceAction << 4), // 4e
			groundState | (invalidSequenceAction << 4), // 4f
			groundState | (invalidSequenceAction << 4), // 50
			groundState | (invalidSequenceAction << 4), // 51
			groundState | (invalidSequenceAction << 4), // 52
			groundState | (invalidSequenceAction << 4), // 53
			groundState | (invalidSequenceAction << 4), // 54
			groundState | (invalidSequenceAction << 4), // 55
			groundState | (invalidSequenceAction << 4), // 56
			groundState | (invalidSequenceAction << 4), // 57
			groundState | (invalidSequenceAction << 4), // 58
			groundState | (invalidSequenceAction << 4), // 59
			groundState | (invalidSequenceAction << 4), // 5a
			groundState | (invalidSequenceAction << 4), // 5b
			groundState | (invalidSequenceAction << 4), // 5c
			groundState | (invalidSequenceAction << 4), // 5d
			groundState | (invalidSequenceAction << 4), // 5e
			groundState | (invalidSequenceAction << 4), // 5f
			groundState | (invalidSequenceAction << 4), // 60
			groundState | (invalidSequenceAction << 4), // 61
			groundState | (invalidSequenceAction << 4), // 62
			groundState | (invalidSequenceAction << 4), // 63
			groundState | (invalidSequenceAction << 4), // 64
			groundState | (invalidSequenceAction << 4), // 65
			groundState | (invalidSequenceAction << 4), // 66
			groundState | (invalidSequenceAction << 4), // 67
			groundState | (invalidSequenceAction << 4), // 68
			groundState | (invalidSequenceAction << 4), // 69
			groundState | (invalidSequenceAction << 4), // 6a
			groundState | (invalidSequenceAction << 4), // 6b
			groundState | (invalidSequenceAction << 4), // 6c
			groundState | (invalidSequenceAction << 4), // 6d
			groundState | (invalidSequenceAction << 4), // 6e
			groundState | (invalidSequenceAction << 4), // 6f
			groundState | (invalidSequenceAction << 4), // 70
			groundState | (invalidSequenceAction << 4), // 71
			groundState | (invalidSequenceAction << 4), // 72
			groundState | (invalidSequenceAction << 4), // 73
			groundState | (invalidSequenceAction << 4), // 74
			groundState | (invalidSequenceAction << 4), // 75
			groundState | (invalidSequenceAction << 4), // 76
			groundState | (invalidSequenceAction << 4), // 77
			groundState | (invalidSequenceAction << 4), // 78
			groundState | (invalidSequenceAction << 4), // 79
			groundState | (invalidSequenceAction << 4), // 7a
			groundState | (invalidSequenceAction << 4), // 7b
			groundState | (invalidSequenceAction << 4), // 7c
			groundState | (invalidSequenceAction << 4), // 7d
			groundState | (invalidSequenceAction << 4), // 7e
			groundState | (invalidSequenceAction << 4), // 7f
			groundState | (setByte1Action << 4),        // 80
			groundState | (setByte1Action << 4),        // 81
			groundState | (setByte1Action << 4),        // 82
			groundState | (setByte1Action << 4),        // 83
			groundState | (setByte1Action << 4),        // 84
			groundState | (setByte1Action << 4),        // 85
			groundState | (setByte1Action << 4),        // 86
			groundState | (setByte1Action << 4),        // 87
			groundState | (setByte1Action << 4),        // 88
			groundState | (setByte1Action << 4),        // 89
			groundState | (setByte1Action << 4),        // 8a
			groundState | (setByte1Action << 4),        // 8b
			groundState | (setByte1Action << 4),        // 8c
			groundState | (setByte1Action << 4),        // 8d
			groundState | (setByte1Action << 4),        // 8e
			groundState | (setByte1Action << 4),        // 8f
			groundState | (setByte1Action << 4),        // 90
			groundState | (setByte1Action << 4),        // 91
			groundState | (setByte1Action << 4),        // 92
			groundState | (setByte1Action << 4),        // 93
			groundState | (setByte1Action << 4),        // 94
			groundState | (setByte1Action << 4),        // 95
			groundState | (setByte1Action << 4),        // 96
			groundState | (setByte1Action << 4),        // 97
			groundState | (setByte1Action << 4),        // 98
			groundState | (setByte1Action << 4),        // 99
			groundState | (setByte1Action << 4),        // 9a
			groundState | (setByte1Action << 4),        // 9b
			groundState | (setByte1Action << 4),        // 9c
			groundState | (setByte1Action << 4),        // 9d
			groundState | (setByte1Action << 4),        // 9e
			groundState | (setByte1Action << 4),        // 9f
			groundState | (setByte1Action << 4),        // a0
			groundState | (setByte1Action << 4),        // a1
			groundState | (setByte1Action << 4),        // a2
			groundState | (setByte1Action << 4),        // a3
			groundState | (setByte1Action << 4),        // a4
			groundState | (setByte1Action << 4),        // a5
			groundState | (setByte1Action << 4),        // a6
			groundState | (setByte1Action << 4),        // a7
			groundState | (setByte1Action << 4),        // a8
			groundState | (setByte1Action << 4),        // a9
			groundState | (setByte1Action << 4),        // aa
			groundState | (setByte1Action << 4),        // ab
			groundState | (setByte1Action << 4),        // ac
			groundState | (setByte1Action << 4),        // ad
			groundState | (setByte1Action << 4),        // ae
			groundState | (setByte1Action << 4),        // af
			groundState | (setByte1Action << 4),        // b0
			groundState | (setByte1Action << 4),        // b1
			groundState | (setByte1Action << 4),        // b2
			groundState | (setByte1Action << 4),        // b3
			groundState | (setByte1Action << 4),        // b4
			groundState | (setByte1Action << 4),        // b5
			groundState | (setByte1Action << 4),        // b6
			groundState | (setByte1Action << 4),        // b7
			groundState | (setByte1Action << 4),        // b8
			groundState | (setByte1Action << 4),        // b9
			groundState | (setByte1Action << 4),        // ba
			groundState | (setByte1Action << 4),        // bb
			groundState | (setByte1Action << 4),        // bc
			groundState | (setByte1Action << 4),        // bd
			groundState | (setByte1Action << 4),        // be
			groundState | (setByte1Action << 4),        // bf
			groundState | (invalidSequenceAction << 4), // c0
			groundState | (invalidSequenceAction << 4), // c1
			groundState | (invalidSequenceAction << 4), // c2
			groundState | (invalidSequenceAction << 4), // c3
			groundState | (invalidSequenceAction << 4), // c4
			groundState | (invalidSequenceAction << 4), // c5
			groundState | (invalidSequenceAction << 4), // c6
			groundState | (invalidSequenceAction << 4), // c7
			groundState | (invalidSequenceAction << 4), // c8
			groundState | (invalidSequenceAction << 4), // c9
			groundState | (invalidSequenceAction << 4), // ca
			groundState | (invalidSequenceAction << 4), // cb
			groundState | (invalidSequenceAction << 4), // cc
			groundState | (invalidSequenceAction << 4), // cd
			groundState | (invalidSequenceAction << 4), // ce
			groundState | (invalidSequenceAction << 4), // cf
			groundState | (invalidSequenceAction << 4), // d0
			groundState | (invalidSequenceAction << 4), // d1
			groundState | (invalidSequenceAction << 4), // d2
			groundState | (invalidSequenceAction << 4), // d3
			groundState | (invalidSequenceAction << 4), // d4
			groundState | (invalidSequenceAction << 4), // d5
			groundState | (invalidSequenceAction << 4), // d6
			groundState | (invalidSequenceAction << 4), // d7
			groundState | (invalidSequenceAction << 4), // d8
			groundState | (invalidSequenceAction << 4), // d9
			groundState | (invalidSequenceAction << 4), // da
			groundState | (invalidSequenceAction << 4), // db
			groundState | (invalidSequenceAction << 4), // dc
			groundState | (invalidSequenceAction << 4), // dd
			groundState | (invalidSequenceAction << 4), // de
			groundState | (invalidSequenceAction << 4), // df
			groundState | (invalidSequenceAction << 4), // e0
			groundState | (invalidSequenceAction << 4), // e1
			groundState | (invalidSequenceAction << 4), // e2
			groundState | (invalidSequenceAction << 4), // e3
			groundState | (invalidSequenceAction << 4), // e4
			groundState | (invalidSequenceAction << 4), // e5
			groundState | (invalidSequenceAction << 4), // e6
			groundState | (invalidSequenceAction << 4), // e7
			groundState | (invalidSequenceAction << 4), // e8
			groundState | (invalidSequenceAction << 4), // e9
			groundState | (invalidSequenceAction << 4), // ea
			groundState | (invalidSequenceAction << 4), // eb
			groundState | (invalidSequenceAction << 4), // ec
			groundState | (invalidSequenceAction << 4), // ed
			groundState | (invalidSequenceAction << 4), // ee
			groundState | (invalidSequenceAction << 4), // ef
			groundState | (invalidSequenceAction << 4), // f0
			groundState | (invalidSequenceAction << 4), // f1
			groundState | (invalidSequenceAction << 4), // f2
			groundState | (invalidSequenceAction << 4), // f3
			groundState | (invalidSequenceAction << 4), // f4
			groundState | (invalidSequenceAction << 4), // f5
			groundState | (invalidSequenceAction << 4), // f6
			groundState | (invalidSequenceAction << 4), // f7
			groundState | (invalidSequenceAction << 4), // f8
			groundState | (invalidSequenceAction << 4), // f9
			groundState | (invalidSequenceAction << 4), // fa
			groundState | (invalidSequenceAction << 4), // fb
			groundState | (invalidSequenceAction << 4), // fc
			groundState | (invalidSequenceAction << 4), // fd
			groundState | (invalidSequenceAction << 4), // fe
			groundState | (invalidSequenceAction << 4), // ff
		},
		{ //State u32e0State = 4
			groundState | (invalidSequenceAction << 4), // 00
			groundState | (invalidSequenceAction << 4), // 01
			groundState | (invalidSequenceAction << 4), // 02
			groundState | (invalidSequenceAction << 4), // 03
			groundState | (invalidSequenceAction << 4), // 04
			groundState | (invalidSequenceAction << 4), // 05
			groundState | (invalidSequenceAction << 4), // 06
			groundState | (invalidSequenceAction << 4), // 07
			groundState | (invalidSequenceAction << 4), // 08
			groundState | (invalidSequenceAction << 4), // 09
			groundState | (invalidSequenceAction << 4), // 0a
			groundState | (invalidSequenceAction << 4), // 0b
			groundState | (invalidSequenceAction << 4), // 0c
			groundState | (invalidSequenceAction << 4), // 0d
			groundState | (invalidSequenceAction << 4), // 0e
			groundState | (invalidSequenceAction << 4), // 0f
			groundState | (invalidSequenceAction << 4), // 10
			groundState | (invalidSequenceAction << 4), // 11
			groundState | (invalidSequenceAction << 4), // 12
			groundState | (invalidSequenceAction << 4), // 13
			groundState | (invalidSequenceAction << 4), // 14
			groundState | (invalidSequenceAction << 4), // 15
			groundState | (invalidSequenceAction << 4), // 16
			groundState | (invalidSequenceAction << 4), // 17
			groundState | (invalidSequenceAction << 4), // 18
			groundState | (invalidSequenceAction << 4), // 19
			groundState | (invalidSequenceAction << 4), // 1a
			groundState | (invalidSequenceAction << 4), // 1b
			groundState | (invalidSequenceAction << 4), // 1c
			groundState | (invalidSequenceAction << 4), // 1d
			groundState | (invalidSequenceAction << 4), // 1e
			groundState | (invalidSequenceAction << 4), // 1f
			groundState | (invalidSequenceAction << 4), // 20
			groundState | (invalidSequenceAction << 4), // 21
			groundState | (invalidSequenceAction << 4), // 22
			groundState | (invalidSequenceAction << 4), // 23
			groundState | (invalidSequenceAction << 4), // 24
			groundState | (invalidSequenceAction << 4), // 25
			groundState | (invalidSequenceAction << 4), // 26
			groundState | (invalidSequenceAction << 4), // 27
			groundState | (invalidSequenceAction << 4), // 28
			groundState | (invalidSequenceAction << 4), // 29
			groundState | (invalidSequenceAction << 4), // 2a
			groundState | (invalidSequenceAction << 4), // 2b
			groundState | (invalidSequenceAction << 4), // 2c
			groundState | (invalidSequenceAction << 4), // 2d
			groundState | (invalidSequenceAction << 4), // 2e
			groundState | (invalidSequenceAction << 4), // 2f
			groundState | (invalidSequenceAction << 4), // 30
			groundState | (invalidSequenceAction << 4), // 31
			groundState | (invalidSequenceAction << 4), // 32
			groundState | (invalidSequenceAction << 4), // 33
			groundState | (invalidSequenceAction << 4), // 34
			groundState | (invalidSequenceAction << 4), // 35
			groundState | (invalidSequenceAction << 4), // 36
			groundState | (invalidSequenceAction << 4), // 37
			groundState | (invalidSequenceAction << 4), // 38
			groundState | (invalidSequenceAction << 4), // 39
			groundState | (invalidSequenceAction << 4), // 3a
			groundState | (invalidSequenceAction << 4), // 3b
			groundState | (invalidSequenceAction << 4), // 3c
			groundState | (invalidSequenceAction << 4), // 3d
			groundState | (invalidSequenceAction << 4), // 3e
			groundState | (invalidSequenceAction << 4), // 3f
			groundState | (invalidSequenceAction << 4), // 40
			groundState | (invalidSequenceAction << 4), // 41
			groundState | (invalidSequenceAction << 4), // 42
			groundState | (invalidSequenceAction << 4), // 43
			groundState | (invalidSequenceAction << 4), // 44
			groundState | (invalidSequenceAction << 4), // 45
			groundState | (invalidSequenceAction << 4), // 46
			groundState | (invalidSequenceAction << 4), // 47
			groundState | (invalidSequenceAction << 4), // 48
			groundState | (invalidSequenceAction << 4), // 49
			groundState | (invalidSequenceAction << 4), // 4a
			groundState | (invalidSequenceAction << 4), // 4b
			groundState | (invalidSequenceAction << 4), // 4c
			groundState | (invalidSequenceAction << 4), // 4d
			groundState | (invalidSequenceAction << 4), // 4e
			groundState | (invalidSequenceAction << 4), // 4f
			groundState | (invalidSequenceAction << 4), // 50
			groundState | (invalidSequenceAction << 4), // 51
			groundState | (invalidSequenceAction << 4), // 52
			groundState | (invalidSequenceAction << 4), // 53
			groundState | (invalidSequenceAction << 4), // 54
			groundState | (invalidSequenceAction << 4), // 55
			groundState | (invalidSequenceAction << 4), // 56
			groundState | (invalidSequenceAction << 4), // 57
			groundState | (invalidSequenceAction << 4), // 58
			groundState | (invalidSequenceAction << 4), // 59
			groundState | (invalidSequenceAction << 4), // 5a
			groundState | (invalidSequenceAction << 4), // 5b
			groundState | (invalidSequenceAction << 4), // 5c
			groundState | (invalidSequenceAction << 4), // 5d
			groundState | (invalidSequenceAction << 4), // 5e
			groundState | (invalidSequenceAction << 4), // 5f
			groundState | (invalidSequenceAction << 4), // 60
			groundState | (invalidSequenceAction << 4), // 61
			groundState | (invalidSequenceAction << 4), // 62
			groundState | (invalidSequenceAction << 4), // 63
			groundState | (invalidSequenceAction << 4), // 64
			groundState | (invalidSequenceAction << 4), // 65
			groundState | (invalidSequenceAction << 4), // 66
			groundState | (invalidSequenceAction << 4), // 67
			groundState | (invalidSequenceAction << 4), // 68
			groundState | (invalidSequenceAction << 4), // 69
			groundState | (invalidSequenceAction << 4), // 6a
			groundState | (invalidSequenceAction << 4), // 6b
			groundState | (invalidSequenceAction << 4), // 6c
			groundState | (invalidSequenceAction << 4), // 6d
			groundState | (invalidSequenceAction << 4), // 6e
			groundState | (invalidSequenceAction << 4), // 6f
			groundState | (invalidSequenceAction << 4), // 70
			groundState | (invalidSequenceAction << 4), // 71
			groundState | (invalidSequenceAction << 4), // 72
			groundState | (invalidSequenceAction << 4), // 73
			groundState | (invalidSequenceAction << 4), // 74
			groundState | (invalidSequenceAction << 4), // 75
			groundState | (invalidSequenceAction << 4), // 76
			groundState | (invalidSequenceAction << 4), // 77
			groundState | (invalidSequenceAction << 4), // 78
			groundState | (invalidSequenceAction << 4), // 79
			groundState | (invalidSequenceAction << 4), // 7a
			groundState | (invalidSequenceAction << 4), // 7b
			groundState | (invalidSequenceAction << 4), // 7c
			groundState | (invalidSequenceAction << 4), // 7d
			groundState | (invalidSequenceAction << 4), // 7e
			groundState | (invalidSequenceAction << 4), // 7f
			groundState | (invalidSequenceAction << 4), // 80
			groundState | (invalidSequenceAction << 4), // 81
			groundState | (invalidSequenceAction << 4), // 82
			groundState | (invalidSequenceAction << 4), // 83
			groundState | (invalidSequenceAction << 4), // 84
			groundState | (invalidSequenceAction << 4), // 85
			groundState | (invalidSequenceAction << 4), // 86
			groundState | (invalidSequenceAction << 4), // 87
			groundState | (invalidSequenceAction << 4), // 88
			groundState | (invalidSequenceAction << 4), // 89
			groundState | (invalidSequenceAction << 4), // 8a
			groundState | (invalidSequenceAction << 4), // 8b
			groundState | (invalidSequenceAction << 4), // 8c
			groundState | (invalidSequenceAction << 4), // 8d
			groundState | (invalidSequenceAction << 4), // 8e
			groundState | (invalidSequenceAction << 4), // 8f
			groundState | (invalidSequenceAction << 4), // 90
			groundState | (invalidSequenceAction << 4), // 91
			groundState | (invalidSequenceAction << 4), // 92
			groundState | (invalidSequenceAction << 4), // 93
			groundState | (invalidSequenceAction << 4), // 94
			groundState | (invalidSequenceAction << 4), // 95
			groundState | (invalidSequenceAction << 4), // 96
			groundState | (invalidSequenceAction << 4), // 97
			groundState | (invalidSequenceAction << 4), // 98
			groundState | (invalidSequenceAction << 4), // 99
			groundState | (invalidSequenceAction << 4), // 9a
			groundState | (invalidSequenceAction << 4), // 9b
			groundState | (invalidSequenceAction << 4), // 9c
			groundState | (invalidSequenceAction << 4), // 9d
			groundState | (invalidSequenceAction << 4), // 9e
			groundState | (invalidSequenceAction << 4), // 9f
			tail1State | (setByte2Action << 4),         // a0
			tail1State | (setByte2Action << 4),         // a1
			tail1State | (setByte2Action << 4),         // a2
			tail1State | (setByte2Action << 4),         // a3
			tail1State | (setByte2Action << 4),         // a4
			tail1State | (setByte2Action << 4),         // a5
			tail1State | (setByte2Action << 4),         // a6
			tail1State | (setByte2Action << 4),         // a7
			tail1State | (setByte2Action << 4),         // a8
			tail1State | (setByte2Action << 4),         // a9
			tail1State | (setByte2Action << 4),         // aa
			tail1State | (setByte2Action << 4),         // ab
			tail1State | (setByte2Action << 4),         // ac
			tail1State | (setByte2Action << 4),         // ad
			tail1State | (setByte2Action << 4),         // ae
			tail1State | (setByte2Action << 4),         // af
			tail1State | (setByte2Action << 4),         // b0
			tail1State | (setByte2Action << 4),         // b1
			tail1State | (setByte2Action << 4),         // b2
			tail1State | (setByte2Action << 4),         // b3
			tail1State | (setByte2Action << 4),         // b4
			tail1State | (setByte2Action << 4),         // b5
			tail1State | (setByte2Action << 4),         // b6
			tail1State | (setByte2Action << 4),         // b7
			tail1State | (setByte2Action << 4),         // b8
			tail1State | (setByte2Action << 4),         // b9
			tail1State | (setByte2Action << 4),         // ba
			tail1State | (setByte2Action << 4),         // bb
			tail1State | (setByte2Action << 4),         // bc
			tail1State | (setByte2Action << 4),         // bd
			tail1State | (setByte2Action << 4),         // be
			tail1State | (setByte2Action << 4),         // bf
			groundState | (invalidSequenceAction << 4), // c0
			groundState | (invalidSequenceAction << 4), // c1
			groundState | (invalidSequenceAction << 4), // c2
			groundState | (invalidSequenceAction << 4), // c3
			groundState | (invalidSequenceAction << 4), // c4
			groundState | (invalidSequenceAction << 4), // c5
			groundState | (invalidSequenceAction << 4), // c6
			groundState | (invalidSequenceAction << 4), // c7
			groundState | (invalidSequenceAction << 4), // c8
			groundState | (invalidSequenceAction << 4), // c9
			groundState | (invalidSequenceAction << 4), // ca
			groundState | (invalidSequenceAction << 4), // cb
			groundState | (invalidSequenceAction << 4), // cc
			groundState | (invalidSequenceAction << 4), // cd
			groundState | (invalidSequenceAction << 4), // ce
			groundState | (invalidSequenceAction << 4), // cf
			groundState | (invalidSequenceAction << 4), // d0
			groundState | (invalidSequenceAction << 4), // d1
			groundState | (invalidSequenceAction << 4), // d2
			groundState | (invalidSequenceAction << 4), // d3
			groundState | (invalidSequenceAction << 4), // d4
			groundState | (invalidSequenceAction << 4), // d5
			groundState | (invalidSequenceAction << 4), // d6
			groundState | (invalidSequenceAction << 4), // d7
			groundState | (invalidSequenceAction << 4), // d8
			groundState | (invalidSequenceAction << 4), // d9
			groundState | (invalidSequenceAction << 4), // da
			groundState | (invalidSequenceAction << 4), // db
			groundState | (invalidSequenceAction << 4), // dc
			groundState | (invalidSequenceAction << 4), // dd
			groundState | (invalidSequenceAction << 4), // de
			groundState | (invalidSequenceAction << 4), // df
			groundState | (invalidSequenceAction << 4), // e0
			groundState | (invalidSequenceAction << 4), // e1
			groundState | (invalidSequenceAction << 4), // e2
			groundState | (invalidSequenceAction << 4), // e3
			groundState | (invalidSequenceAction << 4), // e4
			groundState | (invalidSequenceAction << 4), // e5
			groundState | (invalidSequenceAction << 4), // e6
			groundState | (invalidSequenceAction << 4), // e7
			groundState | (invalidSequenceAction << 4), // e8
			groundState | (invalidSequenceAction << 4), // e9
			groundState | (invalidSequenceAction << 4), // ea
			groundState | (invalidSequenceAction << 4), // eb
			groundState | (invalidSequenceAction << 4), // ec
			groundState | (invalidSequenceAction << 4), // ed
			groundState | (invalidSequenceAction << 4), // ee
			groundState | (invalidSequenceAction << 4), // ef
			groundState | (invalidSequenceAction << 4), // f0
			groundState | (invalidSequenceAction << 4), // f1
			groundState | (invalidSequenceAction << 4), // f2
			groundState | (invalidSequenceAction << 4), // f3
			groundState | (invalidSequenceAction << 4), // f4
			groundState | (invalidSequenceAction << 4), // f5
			groundState | (invalidSequenceAction << 4), // f6
			groundState | (invalidSequenceAction << 4), // f7
			groundState | (invalidSequenceAction << 4), // f8
			groundState | (invalidSequenceAction << 4), // f9
			groundState | (invalidSequenceAction << 4), // fa
			groundState | (invalidSequenceAction << 4), // fb
			groundState | (invalidSequenceAction << 4), // fc
			groundState | (invalidSequenceAction << 4), // fd
			groundState | (invalidSequenceAction << 4), // fe
			groundState | (invalidSequenceAction << 4), // ff
		},
		{ //State u32edState = 5
			groundState | (invalidSequenceAction << 4), // 00
			groundState | (invalidSequenceAction << 4), // 01
			groundState | (invalidSequenceAction << 4), // 02
			groundState | (invalidSequenceAction << 4), // 03
			groundState | (invalidSequenceAction << 4), // 04
			groundState | (invalidSequenceAction << 4), // 05
			groundState | (invalidSequenceAction << 4), // 06
			groundState | (invalidSequenceAction << 4), // 07
			groundState | (invalidSequenceAction << 4), // 08
			groundState | (invalidSequenceAction << 4), // 09
			groundState | (invalidSequenceAction << 4), // 0a
			groundState | (invalidSequenceAction << 4), // 0b
			groundState | (invalidSequenceAction << 4), // 0c
			groundState | (invalidSequenceAction << 4), // 0d
			groundState | (invalidSequenceAction << 4), // 0e
			groundState | (invalidSequenceAction << 4), // 0f
			groundState | (invalidSequenceAction << 4), // 10
			groundState | (invalidSequenceAction << 4), // 11
			groundState | (invalidSequenceAction << 4), // 12
			groundState | (invalidSequenceAction << 4), // 13
			groundState | (invalidSequenceAction << 4), // 14
			groundState | (invalidSequenceAction << 4), // 15
			groundState | (invalidSequenceAction << 4), // 16
			groundState | (invalidSequenceAction << 4), // 17
			groundState | (invalidSequenceAction << 4), // 18
			groundState | (invalidSequenceAction << 4), // 19
			groundState | (invalidSequenceAction << 4), // 1a
			groundState | (invalidSequenceAction << 4), // 1b
			groundState | (invalidSequenceAction << 4), // 1c
			groundState | (invalidSequenceAction << 4), // 1d
			groundState | (invalidSequenceAction << 4), // 1e
			groundState | (invalidSequenceAction << 4), // 1f
			groundState | (invalidSequenceAction << 4), // 20
			groundState | (invalidSequenceAction << 4), // 21
			groundState | (invalidSequenceAction << 4), // 22
			groundState | (invalidSequenceAction << 4), // 23
			groundState | (invalidSequenceAction << 4), // 24
			groundState | (invalidSequenceAction << 4), // 25
			groundState | (invalidSequenceAction << 4), // 26
			groundState | (invalidSequenceAction << 4), // 27
			groundState | (invalidSequenceAction << 4), // 28
			groundState | (invalidSequenceAction << 4), // 29
			groundState | (invalidSequenceAction << 4), // 2a
			groundState | (invalidSequenceAction << 4), // 2b
			groundState | (invalidSequenceAction << 4), // 2c
			groundState | (invalidSequenceAction << 4), // 2d
			groundState | (invalidSequenceAction << 4), // 2e
			groundState | (invalidSequenceAction << 4), // 2f
			groundState | (invalidSequenceAction << 4), // 30
			groundState | (invalidSequenceAction << 4), // 31
			groundState | (invalidSequenceAction << 4), // 32
			groundState | (invalidSequenceAction << 4), // 33
			groundState | (invalidSequenceAction << 4), // 34
			groundState | (invalidSequenceAction << 4), // 35
			groundState | (invalidSequenceAction << 4), // 36
			groundState | (invalidSequenceAction << 4), // 37
			groundState | (invalidSequenceAction << 4), // 38
			groundState | (invalidSequenceAction << 4), // 39
			groundState | (invalidSequenceAction << 4), // 3a
			groundState | (invalidSequenceAction << 4), // 3b
			groundState | (invalidSequenceAction << 4), // 3c
			groundState | (invalidSequenceAction << 4), // 3d
			groundState | (invalidSequenceAction << 4), // 3e
			groundState | (invalidSequenceAction << 4), // 3f
			groundState | (invalidSequenceAction << 4), // 40
			groundState | (invalidSequenceAction << 4), // 41
			groundState | (invalidSequenceAction << 4), // 42
			groundState | (invalidSequenceAction << 4), // 43
			groundState | (invalidSequenceAction << 4), // 44
			groundState | (invalidSequenceAction << 4), // 45
			groundState | (invalidSequenceAction << 4), // 46
			groundState | (invalidSequenceAction << 4), // 47
			groundState | (invalidSequenceAction << 4), // 48
			groundState | (invalidSequenceAction << 4), // 49
			groundState | (invalidSequenceAction << 4), // 4a
			groundState | (invalidSequenceAction << 4), // 4b
			groundState | (invalidSequenceAction << 4), // 4c
			groundState | (invalidSequenceAction << 4), // 4d
			groundState | (invalidSequenceAction << 4), // 4e
			groundState | (invalidSequenceAction << 4), // 4f
			groundState | (invalidSequenceAction << 4), // 50
			groundState | (invalidSequenceAction << 4), // 51
			groundState | (invalidSequenceAction << 4), // 52
			groundState | (invalidSequenceAction << 4), // 53
			groundState | (invalidSequenceAction << 4), // 54
			groundState | (invalidSequenceAction << 4), // 55
			groundState | (invalidSequenceAction << 4), // 56
			groundState | (invalidSequenceAction << 4), // 57
			groundState | (invalidSequenceAction << 4), // 58
			groundState | (invalidSequenceAction << 4), // 59
			groundState | (invalidSequenceAction << 4), // 5a
			groundState | (invalidSequenceAction << 4), // 5b
			groundState | (invalidSequenceAction << 4), // 5c
			groundState | (invalidSequenceAction << 4), // 5d
			groundState | (invalidSequenceAction << 4), // 5e
			groundState | (invalidSequenceAction << 4), // 5f
			groundState | (invalidSequenceAction << 4), // 60
			groundState | (invalidSequenceAction << 4), // 61
			groundState | (invalidSequenceAction << 4), // 62
			groundState | (invalidSequenceAction << 4), // 63
			groundState | (invalidSequenceAction << 4), // 64
			groundState | (invalidSequenceAction << 4), // 65
			groundState | (invalidSequenceAction << 4), // 66
			groundState | (invalidSequenceAction << 4), // 67
			groundState | (invalidSequenceAction << 4), // 68
			groundState | (invalidSequenceAction << 4), // 69
			groundState | (invalidSequenceAction << 4), // 6a
			groundState | (invalidSequenceAction << 4), // 6b
			groundState | (invalidSequenceAction << 4), // 6c
			groundState | (invalidSequenceAction << 4), // 6d
			groundState | (invalidSequenceAction << 4), // 6e
			groundState | (invalidSequenceAction << 4), // 6f
			groundState | (invalidSequenceAction << 4), // 70
			groundState | (invalidSequenceAction << 4), // 71
			groundState | (invalidSequenceAction << 4), // 72
			groundState | (invalidSequenceAction << 4), // 73
			groundState | (invalidSequenceAction << 4), // 74
			groundState | (invalidSequenceAction << 4), // 75
			groundState | (invalidSequenceAction << 4), // 76
			groundState | (invalidSequenceAction << 4), // 77
			groundState | (invalidSequenceAction << 4), // 78
			groundState | (invalidSequenceAction << 4), // 79
			groundState | (invalidSequenceAction << 4), // 7a
			groundState | (invalidSequenceAction << 4), // 7b
			groundState | (invalidSequenceAction << 4), // 7c
			groundState | (invalidSequenceAction << 4), // 7d
			groundState | (invalidSequenceAction << 4), // 7e
			groundState | (invalidSequenceAction << 4), // 7f
			tail1State | (setByte2Action << 4),         // 80
			tail1State | (setByte2Action << 4),         // 81
			tail1State | (setByte2Action << 4),         // 82
			tail1State | (setByte2Action << 4),         // 83
			tail1State | (setByte2Action << 4),         // 84
			tail1State | (setByte2Action << 4),         // 85
			tail1State | (setByte2Action << 4),         // 86
			tail1State | (setByte2Action << 4),         // 87
			tail1State | (setByte2Action << 4),         // 88
			tail1State | (setByte2Action << 4),         // 89
			tail1State | (setByte2Action << 4),         // 8a
			tail1State | (setByte2Action << 4),         // 8b
			tail1State | (setByte2Action << 4),         // 8c
			tail1State | (setByte2Action << 4),         // 8d
			tail1State | (setByte2Action << 4),         // 8e
			tail1State | (setByte2Action << 4),         // 8f
			tail1State | (setByte2Action << 4),         // 90
			tail1State | (setByte2Action << 4),         // 91
			tail1State | (setByte2Action << 4),         // 92
			tail1State | (setByte2Action << 4),         // 93
			tail1State | (setByte2Action << 4),         // 94
			tail1State | (setByte2Action << 4),         // 95
			tail1State | (setByte2Action << 4),         // 96
			tail1State | (setByte2Action << 4),         // 97
			tail1State | (setByte2Action << 4),         // 98
			tail1State | (setByte2Action << 4),         // 99
			tail1State | (setByte2Action << 4),         // 9a
			tail1State | (setByte2Action << 4),         // 9b
			tail1State | (setByte2Action << 4),         // 9c
			tail1State | (setByte2Action << 4),         // 9d
			tail1State | (setByte2Action << 4),         // 9e
			tail1State | (setByte2Action << 4),         // 9f
			groundState | (invalidSequenceAction << 4), // a0
			groundState | (invalidSequenceAction << 4), // a1
			groundState | (invalidSequenceAction << 4), // a2
			groundState | (invalidSequenceAction << 4), // a3
			groundState | (invalidSequenceAction << 4), // a4
			groundState | (invalidSequenceAction << 4), // a5
			groundState | (invalidSequenceAction << 4), // a6
			groundState | (invalidSequenceAction << 4), // a7
			groundState | (invalidSequenceAction << 4), // a8
			groundState | (invalidSequenceAction << 4), // a9
			groundState | (invalidSequenceAction << 4), // aa
			groundState | (invalidSequenceAction << 4), // ab
			groundState | (invalidSequenceAction << 4), // ac
			groundState | (invalidSequenceAction << 4), // ad
			groundState | (invalidSequenceAction << 4), // ae
			groundState | (invalidSequenceAction << 4), // af
			groundState | (invalidSequenceAction << 4), // b0
			groundState | (invalidSequenceAction << 4), // b1
			groundState | (invalidSequenceAction << 4), // b2
			groundState | (invalidSequenceAction << 4), // b3
			groundState | (invalidSequenceAction << 4), // b4
			groundState | (invalidSequenceAction << 4), // b5
			groundState | (invalidSequenceAction << 4), // b6
			groundState | (invalidSequenceAction << 4), // b7
			groundState | (invalidSequenceAction << 4), // b8
			groundState | (invalidSequenceAction << 4), // b9
			groundState | (invalidSequenceAction << 4), // ba
			groundState | (invalidSequenceAction << 4), // bb
			groundState | (invalidSequenceAction << 4), // bc
			groundState | (invalidSequenceAction << 4), // bd
			groundState | (invalidSequenceAction << 4), // be
			groundState | (invalidSequenceAction << 4), // bf
			groundState | (invalidSequenceAction << 4), // c0
			groundState | (invalidSequenceAction << 4), // c1
			groundState | (invalidSequenceAction << 4), // c2
			groundState | (invalidSequenceAction << 4), // c3
			groundState | (invalidSequenceAction << 4), // c4
			groundState | (invalidSequenceAction << 4), // c5
			groundState | (invalidSequenceAction << 4), // c6
			groundState | (invalidSequenceAction << 4), // c7
			groundState | (invalidSequenceAction << 4), // c8
			groundState | (invalidSequenceAction << 4), // c9
			groundState | (invalidSequenceAction << 4), // ca
			groundState | (invalidSequenceAction << 4), // cb
			groundState | (invalidSequenceAction << 4), // cc
			groundState | (invalidSequenceAction << 4), // cd
			groundState | (invalidSequenceAction << 4), // ce
			groundState | (invalidSequenceAction << 4), // cf
			groundState | (invalidSequenceAction << 4), // d0
			groundState | (invalidSequenceAction << 4), // d1
			groundState | (invalidSequenceAction << 4), // d2
			groundState | (invalidSequenceAction << 4), // d3
			groundState | (invalidSequenceAction << 4), // d4
			groundState | (invalidSequenceAction << 4), // d5
			groundState | (invalidSequenceAction << 4), // d6
			groundState | (invalidSequenceAction << 4), // d7
			groundState | (invalidSequenceAction << 4), // d8
			groundState | (invalidSequenceAction << 4), // d9
			groundState | (invalidSequenceAction << 4), // da
			groundState | (invalidSequenceAction << 4), // db
			groundState | (invalidSequenceAction << 4), // dc
			groundState | (invalidSequenceAction << 4), // dd
			groundState | (invalidSequenceAction << 4), // de
			groundState | (invalidSequenceAction << 4), // df
			groundState | (invalidSequenceAction << 4), // e0
			groundState | (invalidSequenceAction << 4), // e1
			groundState | (invalidSequenceAction << 4), // e2
			groundState | (invalidSequenceAction << 4), // e3
			groundState | (invalidSequenceAction << 4), // e4
			groundState | (invalidSequenceAction << 4), // e5
			groundState | (invalidSequenceAction << 4), // e6
			groundState | (invalidSequenceAction << 4), // e7
			groundState | (invalidSequenceAction << 4), // e8
			groundState | (invalidSequenceAction << 4), // e9
			groundState | (invalidSequenceAction << 4), // ea
			groundState | (invalidSequenceAction << 4), // eb
			groundState | (invalidSequenceAction << 4), // ec
			groundState | (invalidSequenceAction << 4), // ed
			groundState | (invalidSequenceAction << 4), // ee
			groundState | (invalidSequenceAction << 4), // ef
			groundState | (invalidSequenceAction << 4), // f0
			groundState | (invalidSequenceAction << 4), // f1
			groundState | (invalidSequenceAction << 4), // f2
			groundState | (invalidSequenceAction << 4), // f3
			groundState | (invalidSequenceAction << 4), // f4
			groundState | (invalidSequenceAction << 4), // f5
			groundState | (invalidSequenceAction << 4), // f6
			groundState | (invalidSequenceAction << 4), // f7
			groundState | (invalidSequenceAction << 4), // f8
			groundState | (invalidSequenceAction << 4), // f9
			groundState | (invalidSequenceAction << 4), // fa
			groundState | (invalidSequenceAction << 4), // fb
			groundState | (invalidSequenceAction << 4), // fc
			groundState | (invalidSequenceAction << 4), // fd
			groundState | (invalidSequenceAction << 4), // fe
			groundState | (invalidSequenceAction << 4), // ff
		},
		{ //State utf843f0State = 6
			groundState | (invalidSequenceAction << 4), // 00
			groundState | (invalidSequenceAction << 4), // 01
			groundState | (invalidSequenceAction << 4), // 02
			groundState | (invalidSequenceAction << 4), // 03
			groundState | (invalidSequenceAction << 4), // 04
			groundState | (invalidSequenceAction << 4), // 05
			groundState | (invalidSequenceAction << 4), // 06
			groundState | (invalidSequenceAction << 4), // 07
			groundState | (invalidSequenceAction << 4), // 08
			groundState | (invalidSequenceAction << 4), // 09
			groundState | (invalidSequenceAction << 4), // 0a
			groundState | (invalidSequenceAction << 4), // 0b
			groundState | (invalidSequenceAction << 4), // 0c
			groundState | (invalidSequenceAction << 4), // 0d
			groundState | (invalidSequenceAction << 4), // 0e
			groundState | (invalidSequenceAction << 4), // 0f
			groundState | (invalidSequenceAction << 4), // 10
			groundState | (invalidSequenceAction << 4), // 11
			groundState | (invalidSequenceAction << 4), // 12
			groundState | (invalidSequenceAction << 4), // 13
			groundState | (invalidSequenceAction << 4), // 14
			groundState | (invalidSequenceAction << 4), // 15
			groundState | (invalidSequenceAction << 4), // 16
			groundState | (invalidSequenceAction << 4), // 17
			groundState | (invalidSequenceAction << 4), // 18
			groundState | (invalidSequenceAction << 4), // 19
			groundState | (invalidSequenceAction << 4), // 1a
			groundState | (invalidSequenceAction << 4), // 1b
			groundState | (invalidSequenceAction << 4), // 1c
			groundState | (invalidSequenceAction << 4), // 1d
			groundState | (invalidSequenceAction << 4), // 1e
			groundState | (invalidSequenceAction << 4), // 1f
			groundState | (invalidSequenceAction << 4), // 20
			groundState | (invalidSequenceAction << 4), // 21
			groundState | (invalidSequenceAction << 4), // 22
			groundState | (invalidSequenceAction << 4), // 23
			groundState | (invalidSequenceAction << 4), // 24
			groundState | (invalidSequenceAction << 4), // 25
			groundState | (invalidSequenceAction << 4), // 26
			groundState | (invalidSequenceAction << 4), // 27
			groundState | (invalidSequenceAction << 4), // 28
			groundState | (invalidSequenceAction << 4), // 29
			groundState | (invalidSequenceAction << 4), // 2a
			groundState | (invalidSequenceAction << 4), // 2b
			groundState | (invalidSequenceAction << 4), // 2c
			groundState | (invalidSequenceAction << 4), // 2d
			groundState | (invalidSequenceAction << 4), // 2e
			groundState | (invalidSequenceAction << 4), // 2f
			groundState | (invalidSequenceAction << 4), // 30
			groundState | (invalidSequenceAction << 4), // 31
			groundState | (invalidSequenceAction << 4), // 32
			groundState | (invalidSequenceAction << 4), // 33
			groundState | (invalidSequenceAction << 4), // 34
			groundState | (invalidSequenceAction << 4), // 35
			groundState | (invalidSequenceAction << 4), // 36
			groundState | (invalidSequenceAction << 4), // 37
			groundState | (invalidSequenceAction << 4), // 38
			groundState | (invalidSequenceAction << 4), // 39
			groundState | (invalidSequenceAction << 4), // 3a
			groundState | (invalidSequenceAction << 4), // 3b
			groundState | (invalidSequenceAction << 4), // 3c
			groundState | (invalidSequenceAction << 4), // 3d
			groundState | (invalidSequenceAction << 4), // 3e
			groundState | (invalidSequenceAction << 4), // 3f
			groundState | (invalidSequenceAction << 4), // 40
			groundState | (invalidSequenceAction << 4), // 41
			groundState | (invalidSequenceAction << 4), // 42
			groundState | (invalidSequenceAction << 4), // 43
			groundState | (invalidSequenceAction << 4), // 44
			groundState | (invalidSequenceAction << 4), // 45
			groundState | (invalidSequenceAction << 4), // 46
			groundState | (invalidSequenceAction << 4), // 47
			groundState | (invalidSequenceAction << 4), // 48
			groundState | (invalidSequenceAction << 4), // 49
			groundState | (invalidSequenceAction << 4), // 4a
			groundState | (invalidSequenceAction << 4), // 4b
			groundState | (invalidSequenceAction << 4), // 4c
			groundState | (invalidSequenceAction << 4), // 4d
			groundState | (invalidSequenceAction << 4), // 4e
			groundState | (invalidSequenceAction << 4), // 4f
			groundState | (invalidSequenceAction << 4), // 50
			groundState | (invalidSequenceAction << 4), // 51
			groundState | (invalidSequenceAction << 4), // 52
			groundState | (invalidSequenceAction << 4), // 53
			groundState | (invalidSequenceAction << 4), // 54
			groundState | (invalidSequenceAction << 4), // 55
			groundState | (invalidSequenceAction << 4), // 56
			groundState | (invalidSequenceAction << 4), // 57
			groundState | (invalidSequenceAction << 4), // 58
			groundState | (invalidSequenceAction << 4), // 59
			groundState | (invalidSequenceAction << 4), // 5a
			groundState | (invalidSequenceAction << 4), // 5b
			groundState | (invalidSequenceAction << 4), // 5c
			groundState | (invalidSequenceAction << 4), // 5d
			groundState | (invalidSequenceAction << 4), // 5e
			groundState | (invalidSequenceAction << 4), // 5f
			groundState | (invalidSequenceAction << 4), // 60
			groundState | (invalidSequenceAction << 4), // 61
			groundState | (invalidSequenceAction << 4), // 62
			groundState | (invalidSequenceAction << 4), // 63
			groundState | (invalidSequenceAction << 4), // 64
			groundState | (invalidSequenceAction << 4), // 65
			groundState | (invalidSequenceAction << 4), // 66
			groundState | (invalidSequenceAction << 4), // 67
			groundState | (invalidSequenceAction << 4), // 68
			groundState | (invalidSequenceAction << 4), // 69
			groundState | (invalidSequenceAction << 4), // 6a
			groundState | (invalidSequenceAction << 4), // 6b
			groundState | (invalidSequenceAction << 4), // 6c
			groundState | (invalidSequenceAction << 4), // 6d
			groundState | (invalidSequenceAction << 4), // 6e
			groundState | (invalidSequenceAction << 4), // 6f
			groundState | (invalidSequenceAction << 4), // 70
			groundState | (invalidSequenceAction << 4), // 71
			groundState | (invalidSequenceAction << 4), // 72
			groundState | (invalidSequenceAction << 4), // 73
			groundState | (invalidSequenceAction << 4), // 74
			groundState | (invalidSequenceAction << 4), // 75
			groundState | (invalidSequenceAction << 4), // 76
			groundState | (invalidSequenceAction << 4), // 77
			groundState | (invalidSequenceAction << 4), // 78
			groundState | (invalidSequenceAction << 4), // 79
			groundState | (invalidSequenceAction << 4), // 7a
			groundState | (invalidSequenceAction << 4), // 7b
			groundState | (invalidSequenceAction << 4), // 7c
			groundState | (invalidSequenceAction << 4), // 7d
			groundState | (invalidSequenceAction << 4), // 7e
			groundState | (invalidSequenceAction << 4), // 7f
			groundState | (invalidSequenceAction << 4), // 80
			groundState | (invalidSequenceAction << 4), // 81
			groundState | (invalidSequenceAction << 4), // 82
			groundState | (invalidSequenceAction << 4), // 83
			groundState | (invalidSequenceAction << 4), // 84
			groundState | (invalidSequenceAction << 4), // 85
			groundState | (invalidSequenceAction << 4), // 86
			groundState | (invalidSequenceAction << 4), // 87
			groundState | (invalidSequenceAction << 4), // 88
			groundState | (invalidSequenceAction << 4), // 89
			groundState | (invalidSequenceAction << 4), // 8a
			groundState | (invalidSequenceAction << 4), // 8b
			groundState | (invalidSequenceAction << 4), // 8c
			groundState | (invalidSequenceAction << 4), // 8d
			groundState | (invalidSequenceAction << 4), // 8e
			groundState | (invalidSequenceAction << 4), // 8f
			tail2State | (setByte3Action << 4),         // 90
			tail2State | (setByte3Action << 4),         // 91
			tail2State | (setByte3Action << 4),         // 92
			tail2State | (setByte3Action << 4),         // 93
			tail2State | (setByte3Action << 4),         // 94
			tail2State | (setByte3Action << 4),         // 95
			tail2State | (setByte3Action << 4),         // 96
			tail2State | (setByte3Action << 4),         // 97
			tail2State | (setByte3Action << 4),         // 98
			tail2State | (setByte3Action << 4),         // 99
			tail2State | (setByte3Action << 4),         // 9a
			tail2State | (setByte3Action << 4),         // 9b
			tail2State | (setByte3Action << 4),         // 9c
			tail2State | (setByte3Action << 4),         // 9d
			tail2State | (setByte3Action << 4),         // 9e
			tail2State | (setByte3Action << 4),         // 9f
			tail2State | (setByte3Action << 4),         // a0
			tail2State | (setByte3Action << 4),         // a1
			tail2State | (setByte3Action << 4),         // a2
			tail2State | (setByte3Action << 4),         // a3
			tail2State | (setByte3Action << 4),         // a4
			tail2State | (setByte3Action << 4),         // a5
			tail2State | (setByte3Action << 4),         // a6
			tail2State | (setByte3Action << 4),         // a7
			tail2State | (setByte3Action << 4),         // a8
			tail2State | (setByte3Action << 4),         // a9
			tail2State | (setByte3Action << 4),         // aa
			tail2State | (setByte3Action << 4),         // ab
			tail2State | (setByte3Action << 4),         // ac
			tail2State | (setByte3Action << 4),         // ad
			tail2State | (setByte3Action << 4),         // ae
			tail2State | (setByte3Action << 4),         // af
			tail2State | (setByte3Action << 4),         // b0
			tail2State | (setByte3Action << 4),         // b1
			tail2State | (setByte3Action << 4),         // b2
			tail2State | (setByte3Action << 4),         // b3
			tail2State | (setByte3Action << 4),         // b4
			tail2State | (setByte3Action << 4),         // b5
			tail2State | (setByte3Action << 4),         // b6
			tail2State | (setByte3Action << 4),         // b7
			tail2State | (setByte3Action << 4),         // b8
			tail2State | (setByte3Action << 4),         // b9
			tail2State | (setByte3Action << 4),         // ba
			tail2State | (setByte3Action << 4),         // bb
			tail2State | (setByte3Action << 4),         // bc
			tail2State | (setByte3Action << 4),         // bd
			tail2State | (setByte3Action << 4),         // be
			tail2State | (setByte3Action << 4),         // bf
			groundState | (invalidSequenceAction << 4), // c0
			groundState | (invalidSequenceAction << 4), // c1
			groundState | (invalidSequenceAction << 4), // c2
			groundState | (invalidSequenceAction << 4), // c3
			groundState | (invalidSequenceAction << 4), // c4
			groundState | (invalidSequenceAction << 4), // c5
			groundState | (invalidSequenceAction << 4), // c6
			groundState | (invalidSequenceAction << 4), // c7
			groundState | (invalidSequenceAction << 4), // c8
			groundState | (invalidSequenceAction << 4), // c9
			groundState | (invalidSequenceAction << 4), // ca
			groundState | (invalidSequenceAction << 4), // cb
			groundState | (invalidSequenceAction << 4), // cc
			groundState | (invalidSequenceAction << 4), // cd
			groundState | (invalidSequenceAction << 4), // ce
			groundState | (invalidSequenceAction << 4), // cf
			groundState | (invalidSequenceAction << 4), // d0
			groundState | (invalidSequenceAction << 4), // d1
			groundState | (invalidSequenceAction << 4), // d2
			groundState | (invalidSequenceAction << 4), // d3
			groundState | (invalidSequenceAction << 4), // d4
			groundState | (invalidSequenceAction << 4), // d5
			groundState | (invalidSequenceAction << 4), // d6
			groundState | (invalidSequenceAction << 4), // d7
			groundState | (invalidSequenceAction << 4), // d8
			groundState | (invalidSequenceAction << 4), // d9
			groundState | (invalidSequenceAction << 4), // da
			groundState | (invalidSequenceAction << 4), // db
			groundState | (invalidSequenceAction << 4), // dc
			groundState | (invalidSequenceAction << 4), // dd
			groundState | (invalidSequenceAction << 4), // de
			groundState | (invalidSequenceAction << 4), // df
			groundState | (invalidSequenceAction << 4), // e0
			groundState | (invalidSequenceAction << 4), // e1
			groundState | (invalidSequenceAction << 4), // e2
			groundState | (invalidSequenceAction << 4), // e3
			groundState | (invalidSequenceAction << 4), // e4
			groundState | (invalidSequenceAction << 4), // e5
			groundState | (invalidSequenceAction << 4), // e6
			groundState | (invalidSequenceAction << 4), // e7
			groundState | (invalidSequenceAction << 4), // e8
			groundState | (invalidSequenceAction << 4), // e9
			groundState | (invalidSequenceAction << 4), // ea
			groundState | (invalidSequenceAction << 4), // eb
			groundState | (invalidSequenceAction << 4), // ec
			groundState | (invalidSequenceAction << 4), // ed
			groundState | (invalidSequenceAction << 4), // ee
			groundState | (invalidSequenceAction << 4), // ef
			groundState | (invalidSequenceAction << 4), // f0
			groundState | (invalidSequenceAction << 4), // f1
			groundState | (invalidSequenceAction << 4), // f2
			groundState | (invalidSequenceAction << 4), // f3
			groundState | (invalidSequenceAction << 4), // f4
			groundState | (invalidSequenceAction << 4), // f5
			groundState | (invalidSequenceAction << 4), // f6
			groundState | (invalidSequenceAction << 4), // f7
			groundState | (invalidSequenceAction << 4), // f8
			groundState | (invalidSequenceAction << 4), // f9
			groundState | (invalidSequenceAction << 4), // fa
			groundState | (invalidSequenceAction << 4), // fb
			groundState | (invalidSequenceAction << 4), // fc
			groundState | (invalidSequenceAction << 4), // fd
			groundState | (invalidSequenceAction << 4), // fe
			groundState | (invalidSequenceAction << 4), // ff
		},
		{ //State utf843f4State = 7
			groundState | (invalidSequenceAction << 4), // 00
			groundState | (invalidSequenceAction << 4), // 01
			groundState | (invalidSequenceAction << 4), // 02
			groundState | (invalidSequenceAction << 4), // 03
			groundState | (invalidSequenceAction << 4), // 04
			groundState | (invalidSequenceAction << 4), // 05
			groundState | (invalidSequenceAction << 4), // 06
			groundState | (invalidSequenceAction << 4), // 07
			groundState | (invalidSequenceAction << 4), // 08
			groundState | (invalidSequenceAction << 4), // 09
			groundState | (invalidSequenceAction << 4), // 0a
			groundState | (invalidSequenceAction << 4), // 0b
			groundState | (invalidSequenceAction << 4), // 0c
			groundState | (invalidSequenceAction << 4), // 0d
			groundState | (invalidSequenceAction << 4), // 0e
			groundState | (invalidSequenceAction << 4), // 0f
			groundState | (invalidSequenceAction << 4), // 10
			groundState | (invalidSequenceAction << 4), // 11
			groundState | (invalidSequenceAction << 4), // 12
			groundState | (invalidSequenceAction << 4), // 13
			groundState | (invalidSequenceAction << 4), // 14
			groundState | (invalidSequenceAction << 4), // 15
			groundState | (invalidSequenceAction << 4), // 16
			groundState | (invalidSequenceAction << 4), // 17
			groundState | (invalidSequenceAction << 4), // 18
			groundState | (invalidSequenceAction << 4), // 19
			groundState | (invalidSequenceAction << 4), // 1a
			groundState | (invalidSequenceAction << 4), // 1b
			groundState | (invalidSequenceAction << 4), // 1c
			groundState | (invalidSequenceAction << 4), // 1d
			groundState | (invalidSequenceAction << 4), // 1e
			groundState | (invalidSequenceAction << 4), // 1f
			groundState | (invalidSequenceAction << 4), // 20
			groundState | (invalidSequenceAction << 4), // 21
			groundState | (invalidSequenceAction << 4), // 22
			groundState | (invalidSequenceAction << 4), // 23
			groundState | (invalidSequenceAction << 4), // 24
			groundState | (invalidSequenceAction << 4), // 25
			groundState | (invalidSequenceAction << 4), // 26
			groundState | (invalidSequenceAction << 4), // 27
			groundState | (invalidSequenceAction << 4), // 28
			groundState | (invalidSequenceAction << 4), // 29
			groundState | (invalidSequenceAction << 4), // 2a
			groundState | (invalidSequenceAction << 4), // 2b
			groundState | (invalidSequenceAction << 4), // 2c
			groundState | (invalidSequenceAction << 4), // 2d
			groundState | (invalidSequenceAction << 4), // 2e
			groundState | (invalidSequenceAction << 4), // 2f
			groundState | (invalidSequenceAction << 4), // 30
			groundState | (invalidSequenceAction << 4), // 31
			groundState | (invalidSequenceAction << 4), // 32
			groundState | (invalidSequenceAction << 4), // 33
			groundState | (invalidSequenceAction << 4), // 34
			groundState | (invalidSequenceAction << 4), // 35
			groundState | (invalidSequenceAction << 4), // 36
			groundState | (invalidSequenceAction << 4), // 37
			groundState | (invalidSequenceAction << 4), // 38
			groundState | (invalidSequenceAction << 4), // 39
			groundState | (invalidSequenceAction << 4), // 3a
			groundState | (invalidSequenceAction << 4), // 3b
			groundState | (invalidSequenceAction << 4), // 3c
			groundState | (invalidSequenceAction << 4), // 3d
			groundState | (invalidSequenceAction << 4), // 3e
			groundState | (invalidSequenceAction << 4), // 3f
			groundState | (invalidSequenceAction << 4), // 40
			groundState | (invalidSequenceAction << 4), // 41
			groundState | (invalidSequenceAction << 4), // 42
			groundState | (invalidSequenceAction << 4), // 43
			groundState | (invalidSequenceAction << 4), // 44
			groundState | (invalidSequenceAction << 4), // 45
			groundState | (invalidSequenceAction << 4), // 46
			groundState | (invalidSequenceAction << 4), // 47
			groundState | (invalidSequenceAction << 4), // 48
			groundState | (invalidSequenceAction << 4), // 49
			groundState | (invalidSequenceAction << 4), // 4a
			groundState | (invalidSequenceAction << 4), // 4b
			groundState | (invalidSequenceAction << 4), // 4c
			groundState | (invalidSequenceAction << 4), // 4d
			groundState | (invalidSequenceAction << 4), // 4e
			groundState | (invalidSequenceAction << 4), // 4f
			groundState | (invalidSequenceAction << 4), // 50
			groundState | (invalidSequenceAction << 4), // 51
			groundState | (invalidSequenceAction << 4), // 52
			groundState | (invalidSequenceAction << 4), // 53
			groundState | (invalidSequenceAction << 4), // 54
			groundState | (invalidSequenceAction << 4), // 55
			groundState | (invalidSequenceAction << 4), // 56
			groundState | (invalidSequenceAction << 4), // 57
			groundState | (invalidSequenceAction << 4), // 58
			groundState | (invalidSequenceAction << 4), // 59
			groundState | (invalidSequenceAction << 4), // 5a
			groundState | (invalidSequenceAction << 4), // 5b
			groundState | (invalidSequenceAction << 4), // 5c
			groundState | (invalidSequenceAction << 4), // 5d
			groundState | (invalidSequenceAction << 4), // 5e
			groundState | (invalidSequenceAction << 4), // 5f
			groundState | (invalidSequenceAction << 4), // 60
			groundState | (invalidSequenceAction << 4), // 61
			groundState | (invalidSequenceAction << 4), // 62
			groundState | (invalidSequenceAction << 4), // 63
			groundState | (invalidSequenceAction << 4), // 64
			groundState | (invalidSequenceAction << 4), // 65
			groundState | (invalidSequenceAction << 4), // 66
			groundState | (invalidSequenceAction << 4), // 67
			groundState | (invalidSequenceAction << 4), // 68
			groundState | (invalidSequenceAction << 4), // 69
			groundState | (invalidSequenceAction << 4), // 6a
			groundState | (invalidSequenceAction << 4), // 6b
			groundState | (invalidSequenceAction << 4), // 6c
			groundState | (invalidSequenceAction << 4), // 6d
			groundState | (invalidSequenceAction << 4), // 6e
			groundState | (invalidSequenceAction << 4), // 6f
			groundState | (invalidSequenceAction << 4), // 70
			groundState | (invalidSequenceAction << 4), // 71
			groundState | (invalidSequenceAction << 4), // 72
			groundState | (invalidSequenceAction << 4), // 73
			groundState | (invalidSequenceAction << 4), // 74
			groundState | (invalidSequenceAction << 4), // 75
			groundState | (invalidSequenceAction << 4), // 76
			groundState | (invalidSequenceAction << 4), // 77
			groundState | (invalidSequenceAction << 4), // 78
			groundState | (invalidSequenceAction << 4), // 79
			groundState | (invalidSequenceAction << 4), // 7a
			groundState | (invalidSequenceAction << 4), // 7b
			groundState | (invalidSequenceAction << 4), // 7c
			groundState | (invalidSequenceAction << 4), // 7d
			groundState | (invalidSequenceAction << 4), // 7e
			groundState | (invalidSequenceAction << 4), // 7f
			tail2State | (setByte3Action << 4),         // 80
			tail2State | (setByte3Action << 4),         // 81
			tail2State | (setByte3Action << 4),         // 82
			tail2State | (setByte3Action << 4),         // 83
			tail2State | (setByte3Action << 4),         // 84
			tail2State | (setByte3Action << 4),         // 85
			tail2State | (setByte3Action << 4),         // 86
			tail2State | (setByte3Action << 4),         // 87
			tail2State | (setByte3Action << 4),         // 88
			tail2State | (setByte3Action << 4),         // 89
			tail2State | (setByte3Action << 4),         // 8a
			tail2State | (setByte3Action << 4),         // 8b
			tail2State | (setByte3Action << 4),         // 8c
			tail2State | (setByte3Action << 4),         // 8d
			tail2State | (setByte3Action << 4),         // 8e
			tail2State | (setByte3Action << 4),         // 8f
			groundState | (invalidSequenceAction << 4), // 90
			groundState | (invalidSequenceAction << 4), // 91
			groundState | (invalidSequenceAction << 4), // 92
			groundState | (invalidSequenceAction << 4), // 93
			groundState | (invalidSequenceAction << 4), // 94
			groundState | (invalidSequenceAction << 4), // 95
			groundState | (invalidSequenceAction << 4), // 96
			groundState | (invalidSequenceAction << 4), // 97
			groundState | (invalidSequenceAction << 4), // 98
			groundState | (invalidSequenceAction << 4), // 99
			groundState | (invalidSequenceAction << 4), // 9a
			groundState | (invalidSequenceAction << 4), // 9b
			groundState | (invalidSequenceAction << 4), // 9c
			groundState | (invalidSequenceAction << 4), // 9d
			groundState | (invalidSequenceAction << 4), // 9e
			groundState | (invalidSequenceAction << 4), // 9f
			groundState | (invalidSequenceAction << 4), // a0
			groundState | (invalidSequenceAction << 4), // a1
			groundState | (invalidSequenceAction << 4), // a2
			groundState | (invalidSequenceAction << 4), // a3
			groundState | (invalidSequenceAction << 4), // a4
			groundState | (invalidSequenceAction << 4), // a5
			groundState | (invalidSequenceAction << 4), // a6
			groundState | (invalidSequenceAction << 4), // a7
			groundState | (invalidSequenceAction << 4), // a8
			groundState | (invalidSequenceAction << 4), // a9
			groundState | (invalidSequenceAction << 4), // aa
			groundState | (invalidSequenceAction << 4), // ab
			groundState | (invalidSequenceAction << 4), // ac
			groundState | (invalidSequenceAction << 4), // ad
			groundState | (invalidSequenceAction << 4), // ae
			groundState | (invalidSequenceAction << 4), // af
			groundState | (invalidSequenceAction << 4), // b0
			groundState | (invalidSequenceAction << 4), // b1
			groundState | (invalidSequenceAction << 4), // b2
			groundState | (invalidSequenceAction << 4), // b3
			groundState | (invalidSequenceAction << 4), // b4
			groundState | (invalidSequenceAction << 4), // b5
			groundState | (invalidSequenceAction << 4), // b6
			groundState | (invalidSequenceAction << 4), // b7
			groundState | (invalidSequenceAction << 4), // b8
			groundState | (invalidSequenceAction << 4), // b9
			groundState | (invalidSequenceAction << 4), // ba
			groundState | (invalidSequenceAction << 4), // bb
			groundState | (invalidSequenceAction << 4), // bc
			groundState | (invalidSequenceAction << 4), // bd
			groundState | (invalidSequenceAction << 4), // be
			groundState | (invalidSequenceAction << 4), // bf
			groundState | (invalidSequenceAction << 4), // c0
			groundState | (invalidSequenceAction << 4), // c1
			groundState | (invalidSequenceAction << 4), // c2
			groundState | (invalidSequenceAction << 4), // c3
			groundState | (invalidSequenceAction << 4), // c4
			groundState | (invalidSequenceAction << 4), // c5
			groundState | (invalidSequenceAction << 4), // c6
			groundState | (invalidSequenceAction << 4), // c7
			groundState | (invalidSequenceAction << 4), // c8
			groundState | (invalidSequenceAction << 4), // c9
			groundState | (invalidSequenceAction << 4), // ca
			groundState | (invalidSequenceAction << 4), // cb
			groundState | (invalidSequenceAction << 4), // cc
			groundState | (invalidSequenceAction << 4), // cd
			groundState | (invalidSequenceAction << 4), // ce
			groundState | (invalidSequenceAction << 4), // cf
			groundState | (invalidSequenceAction << 4), // d0
			groundState | (invalidSequenceAction << 4), // d1
			groundState | (invalidSequenceAction << 4), // d2
			groundState | (invalidSequenceAction << 4), // d3
			groundState | (invalidSequenceAction << 4), // d4
			groundState | (invalidSequenceAction << 4), // d5
			groundState | (invalidSequenceAction << 4), // d6
			groundState | (invalidSequenceAction << 4), // d7
			groundState | (invalidSequenceAction << 4), // d8
			groundState | (invalidSequenceAction << 4), // d9
			groundState | (invalidSequenceAction << 4), // da
			groundState | (invalidSequenceAction << 4), // db
			groundState | (invalidSequenceAction << 4), // dc
			groundState | (invalidSequenceAction << 4), // dd
			groundState | (invalidSequenceAction << 4), // de
			groundState | (invalidSequenceAction << 4), // df
			groundState | (invalidSequenceAction << 4), // e0
			groundState | (invalidSequenceAction << 4), // e1
			groundState | (invalidSequenceAction << 4), // e2
			groundState | (invalidSequenceAction << 4), // e3
			groundState | (invalidSequenceAction << 4), // e4
			groundState | (invalidSequenceAction << 4), // e5
			groundState | (invalidSequenceAction << 4), // e6
			groundState | (invalidSequenceAction << 4), // e7
			groundState | (invalidSequenceAction << 4), // e8
			groundState | (invalidSequenceAction << 4), // e9
			groundState | (invalidSequenceAction << 4), // ea
			groundState | (invalidSequenceAction << 4), // eb
			groundState | (invalidSequenceAction << 4), // ec
			groundState | (invalidSequenceAction << 4), // ed
			groundState | (invalidSequenceAction << 4), // ee
			groundState | (invalidSequenceAction << 4), // ef
			groundState | (invalidSequenceAction << 4), // f0
			groundState | (invalidSequenceAction << 4), // f1
			groundState | (invalidSequenceAction << 4), // f2
			groundState | (invalidSequenceAction << 4), // f3
			groundState | (invalidSequenceAction << 4), // f4
			groundState | (invalidSequenceAction << 4), // f5
			groundState | (invalidSequenceAction << 4), // f6
			groundState | (invalidSequenceAction << 4), // f7
			groundState | (invalidSequenceAction << 4), // f8
			groundState | (invalidSequenceAction << 4), // f9
			groundState | (invalidSequenceAction << 4), // fa
			groundState | (invalidSequenceAction << 4), // fb
			groundState | (invalidSequenceAction << 4), // fc
			groundState | (invalidSequenceAction << 4), // fd
			groundState | (invalidSequenceAction << 4), // fe
			groundState | (invalidSequenceAction << 4), // ff
		},
	}
)
