// Code generated by "stringer -type=ValueKind"; DO NOT EDIT.

package expand

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[Unknown-0]
	_ = x[String-1]
	_ = x[NameRef-2]
	_ = x[Indexed-3]
	_ = x[Associative-4]
	_ = x[KeepValue-5]
}

const _ValueKind_name = "UnknownStringNameRefIndexedAssociativeKeepValue"

var _ValueKind_index = [...]uint8{0, 7, 13, 20, 27, 38, 47}

func (i ValueKind) String() string {
	idx := int(i) - 0
	if i < 0 || idx >= len(_ValueKind_index)-1 {
		return "ValueKind(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _ValueKind_name[_ValueKind_index[idx]:_ValueKind_index[idx+1]]
}
