// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package secretmanager

import (
	"cloud.google.com/go/iam"
)

// IAM returns a handle to inspect and change permissions of the resource
// indicated by the given resource path. Name should be of the format
// `projects/my-project/secrets/my-secret`.
func (c *Client) IAM(name string) *iam.Handle {
	if grpcClient, ok := c.internalClient.(*gRPCClient); ok {
		return iam.InternalNewHandleGRPCClient(grpcClient.client, name)
	}
	return iam.InternalNewHandleGRPCClient(nil, name)
}
